@@ -3,22 +3,21 @@
  *
  *	Created by:	Robert French
  *
- *	$Source: /mit/zephyr/repository/zephyr/clients/zctl/zctl.c,v $
- *	$Author: ghudson $
+ *	$Id: zctl.c 2592 2010-08-24 07:08:36Z kcr@ATHENA.MIT.EDU $
  *
  *	Copyright (c) 1987,1988 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
- *	"mit-copyright.h". 
+ *	"mit-copyright.h".
  */
 
 #include <sysdep.h>
 #include <zephyr/zephyr.h>
 #include <ss/ss.h>
-#include <com_err.h>
 #include <pwd.h>
 #include <netdb.h>
+#include <arpa/nameser.h>
 #ifndef lint
-static const char *rcsid_zctl_c = "$Id: zctl.c,v 1.25 1995/07/07 21:55:40 ghudson Exp $";
+static const char rcsid_zctl_c[] = "$Id: zctl.c 2592 2010-08-24 07:08:36Z kcr@ATHENA.MIT.EDU $";
 #endif
 
 #define SUBSATONCE 7
@@ -41,70 +40,91 @@ static const char *rcsid_zctl_c = "$Id: zctl.c,v 1.25 1995/07/07 21:55:40 ghudso
 #define	ERR		(-1)
 #define	NOT_REMOVED	0
 #define	REMOVED		1
-int purge_subs();
 
 int sci_idx;
 char subsname[BUFSIZ];
-char ourhost[MAXHOSTNAMELEN],ourhostcanon[MAXHOSTNAMELEN];
+char ourhost[NS_MAXDNAME], ourhostcanon[NS_MAXDNAME];
 
 extern ss_request_table zctl_cmds;
 
-main(argc,argv)
-	int argc;
-	char *argv[];
+int purge_subs(register ZSubscription_t *, int);
+void add_file(short, ZSubscription_t *, int);
+void del_file(short, ZSubscription_t *, int);
+void fix_macros(ZSubscription_t *, ZSubscription_t *, int);
+void fix_macros2(char *, char **);
+int make_exist(char *);
+char *whoami;
+
+int
+main(int argc,
+     char *argv[])
 {
 	struct passwd *pwd;
 	struct hostent *hent;
-	char ssline[BUFSIZ],oldsubsname[BUFSIZ],*envptr;
+	char ssline[BUFSIZ],oldsubsname[BUFSIZ],*envptr,*tty = NULL;
 	int retval,code,i;
 #ifdef HAVE_SYS_UTSNAME
 	struct utsname name;
 #endif
 
+	whoami = argv[0];
 	if ((retval = ZInitialize()) != ZERR_NONE) {
 		com_err(argv[0],retval,"while initializing");
 		exit (1);
 	}
 
-	envptr = (char *)getenv("HOME");
-	if (envptr)
-		(void) strcpy(subsname,envptr);
-	else {
-		if (!(pwd = getpwuid((int) getuid()))) {
-			fprintf(stderr,"Who are you?\n");
-			exit (1);
-		}
+	/* Set hostname and tty for locations.  If we support X, use the
+	 * DISPLAY environment variable for the tty name. */
+#ifndef X_DISPLAY_MISSING
+	tty = getenv("DISPLAY");
+#endif
+	if ((retval = ZInitLocationInfo(NULL, tty)) != ZERR_NONE)
+	    com_err(argv[0], retval, "initializing location information");
 
-		(void) strcpy(subsname,pwd->pw_dir);
-	}
-	(void) strcpy(oldsubsname,subsname);
-	(void) strcat(oldsubsname,OLD_SUBS);
-	(void) strcat(subsname,USERS_SUBS);
-	if (!access(oldsubsname,F_OK) && access(subsname, F_OK)) {
-		/* only if old one exists and new one does not exist */
-		printf("The .subscriptions file in your home directory is now being used as\n.zephyr.subs . I will rename it to .zephyr.subs for you.\n");
-		if (rename(oldsubsname,subsname))
-			com_err(argv[0], errno, "renaming .subscriptions");
+	envptr = getenv("ZEPHYR_SUBS");
+	if (envptr)
+		strcpy(subsname,envptr);
+	else {
+		envptr = getenv("HOME");
+		if (envptr)
+			strcpy(subsname,envptr);
+		else {
+			if (!(pwd = getpwuid((int) getuid()))) {
+				fprintf(stderr,"Who are you?\n");
+				exit (1);
+			}
+
+			strcpy(subsname,pwd->pw_dir);
+		}
+		strcpy(oldsubsname,subsname);
+		strcat(oldsubsname,OLD_SUBS);
+		strcat(subsname,USERS_SUBS);
+		if (!access(oldsubsname,F_OK) && access(subsname, F_OK)) {
+			/* only if old one exists and new one does not exist */
+			printf("The .subscriptions file in your home directory is now being used as\n.zephyr.subs . I will rename it to .zephyr.subs for you.\n");
+			if (rename(oldsubsname,subsname))
+				com_err(argv[0], errno, "renaming .subscriptions");
+		}
 	}
 
 #ifdef HAVE_SYS_UTSNAME
 	uname(&name);
 	strcpy(ourhost, name.nodename);
 #else
-	if (gethostname(ourhost,MAXHOSTNAMELEN) == -1) {
+	if (gethostname(ourhost, sizeof(ourhost)) == -1) {
 		com_err(argv[0],errno,"while getting host name");
 		exit (1);
 	}
 #endif
 
 	if (!(hent = gethostbyname(ourhost))) {
-		fprintf(stderr,"%s: Can't get canonical name for host %s",
-			argv[0], ourhost);
-		exit (1);
-	}
+		fprintf(stderr,"%s: Can't resolve hostname %s; %s may be "
+			"wrong in subscriptions\n",argv[0],ourhost,
+			TOKEN_CANONNAME);
+		strncpy(ourhostcanon,ourhost,sizeof(ourhostcanon)-1);
+	} else
+		strncpy(ourhostcanon,hent->h_name,sizeof(ourhostcanon)-1);
 
-	(void) strcpy(ourhostcanon,hent->h_name);
-	
 	sci_idx = ss_create_invocation("zctl","",0,&zctl_cmds,&code);
 	if (code) {
 		ss_perror(sci_idx,code,"while creating invocation");
@@ -121,19 +141,19 @@ main(argc,argv)
 		    fprintf (stderr, "%s: %s: %s\n",
 			     argv[0], error_message (code), ssline);
 		exit((code != 0));
-	} 
+	}
 
-	printf("ZCTL $Revision: 1.25 $ (Protocol %s%d.%d) - Type '?' for a list of commands.\n\n",
+	printf("ZCTL $Revision: 2592 $ (Protocol %s%d.%d) - Type '?' for a list of commands.\n\n",
 	       ZVERSIONHDR,
 	       ZVERSIONMAJOR,ZVERSIONMINOR);
-	
-	ss_listen(sci_idx,&code);
+
+	ss_listen(sci_idx);
 	exit(0);
 }
 
-set_file(argc,argv)
-	int argc;
-	char *argv[];
+void
+set_file(int argc,
+	 char *argv[])
 {
 	if (argc > 2) {
 		fprintf(stderr,"Usage: %s filename\n",argv[0]);
@@ -146,24 +166,24 @@ set_file(argc,argv)
 		(void) strcpy(subsname,argv[1]);
 }
 
-flush_locations(argc,argv)
-	int argc;
-	char *argv[];
+void
+flush_locations(int argc,
+		char *argv[])
 {
 	int retval;
-	
+
 	if (argc > 1) {
 		fprintf(stderr,"Usage: %s\n",argv[0]);
 		return;
 	}
 
 	if ((retval = ZFlushMyLocations()) != ZERR_NONE)
-		ss_perror(sci_idx,retval,"while flushing locations");
+		com_err(whoami, retval, "while flushing locations");
 }
 
-wgc_control(argc,argv)
-	int argc;
-	register char **argv;
+void
+wgc_control(int argc,
+	    char *argv[])
 {
 	int retval;
 	short newport;
@@ -176,15 +196,15 @@ wgc_control(argc,argv)
 		fprintf(stderr,"Usage: %s\n",argv[0]);
 		return;
 	}
-	
+
 	if ((newport = ZGetWGPort()) == -1) {
-		ss_perror(sci_idx,errno,"while getting WindowGram port");
+		com_err(whoami, errno, "while getting WindowGram port");
 		return;
 	}
 
 	newsin.sin_port = (u_short) newport;
 	if ((retval = ZSetDestAddr(&newsin)) != ZERR_NONE) {
-		ss_perror(sci_idx,retval,"while setting destination address");
+		com_err(whoami, retval, "while setting destination address");
 		return;
 	}
 
@@ -200,6 +220,8 @@ wgc_control(argc,argv)
 		notice.z_opcode = USER_SHUTDOWN;
 	if (!strcmp(argv[0],"wg_startup"))
 		notice.z_opcode = USER_STARTUP;
+	if (!strcmp(argv[0],"wg_exit"))
+		notice.z_opcode = USER_EXIT;
 	if (!notice.z_opcode) {
 		fprintf(stderr,
 			"unknown WindowGram client control command %s\n",
@@ -212,16 +234,16 @@ wgc_control(argc,argv)
 	notice.z_message_len = 0;
 
 	if ((retval = ZSendNotice(&notice,ZNOAUTH)) != ZERR_NONE)
-		ss_perror(sci_idx,retval,"while sending notice");
+		com_err(whoami, retval, "while sending notice");
 
 	if ((retval = ZInitialize()) != ZERR_NONE)
-		ss_perror(sci_idx,retval,
+		com_err(whoami, retval,
 			  "while reinitializing");
-} 
+}
 
-hm_control(argc,argv)
-	int argc;
-	char *argv[];
+void
+hm_control(int argc,
+	   char *argv[])
 {
 	int retval;
 	ZNotice_t notice;
@@ -230,7 +252,7 @@ hm_control(argc,argv)
 		fprintf(stderr,"Usage: %s\n",argv[0]);
 		return;
 	}
-	
+
 	(void) memset((char *)&notice, 0, sizeof(notice));
 	notice.z_kind = HMCTL;
 	notice.z_port = 0;
@@ -252,16 +274,16 @@ hm_control(argc,argv)
 	notice.z_message_len = 0;
 
 	if ((retval = ZSendNotice(&notice,ZNOAUTH)) != ZERR_NONE)
-		ss_perror(sci_idx,retval,"while sending notice");
+		com_err(whoami, retval, "while sending notice");
 } 
 
-show_var(argc,argv)
-	int argc;
-	char *argv[];
+void
+show_var(int argc,
+	 char *argv[])
 {
 	int i;
 	char *value;
-	
+
 	if (argc < 2) {
 		fprintf(stderr,"Usage: %s <varname> <varname> ...\n",argv[0]);
 		return;
@@ -276,14 +298,14 @@ show_var(argc,argv)
 	}
 }
 
-set_var(argc,argv)
-	int argc;
-	register char **argv;
+void
+set_var(int argc, char *argv[])
 {
 	int retval,setting_exp,i;
-	char *exp_level,*newargv[1];
+	char *exp_level = NULL;
+	char *newargv[1];
 	char varcat[BUFSIZ];
-	
+
 	if (argc < 2) {
 		fprintf(stderr,"Usage: %s <varname> [value]\n",
 			argv[0]);
@@ -322,7 +344,7 @@ set_var(argc,argv)
 				EXPOSE_NETANN);
 			return;
 		}
-	} 
+	}
 	if (argc == 2)
 		retval = ZSetVariable(argv[1],"");
 	else {
@@ -330,20 +352,20 @@ set_var(argc,argv)
 		for (i=3;i<argc;i++) {
 			(void) strcat(varcat," ");
 			(void) strcat(varcat,argv[i]);
-		} 
+		}
 		retval = ZSetVariable(argv[1],varcat);
-	} 
+	}
 
 	if (retval != ZERR_NONE) {
-		ss_perror(sci_idx,retval,"while setting variable value");
+		com_err(whoami, retval, "while setting variable value");
 		return;
 	}
 
 	/* Side-effects?  Naw, us? */
-	
+
 	if (setting_exp) {
 		if ((retval = ZSetLocation(exp_level)) != ZERR_NONE)
-			ss_perror(sci_idx,retval,"while changing exposure status");
+			com_err(whoami, retval, "while changing exposure status");
 		if (!strcmp(exp_level,EXPOSE_NONE)) {
 			newargv[0] = "wg_shutdown";
 			wgc_control(1,newargv);
@@ -352,12 +374,12 @@ set_var(argc,argv)
 			wgc_control(1,newargv);
 		}
 		return;
-	} 
+	}
 }
 
-do_hide(argc,argv)
-	int argc;
-	char *argv[];
+void
+do_hide(int argc,
+	char *argv[])
 {
 	char *exp_level = NULL;
 	Code_t retval;
@@ -371,16 +393,16 @@ do_hide(argc,argv)
 	else
 		exp_level = EXPOSE_OPSTAFF;
 	if ((retval = ZSetLocation(exp_level)) != ZERR_NONE)
-		ss_perror(sci_idx,retval,"while changing exposure status");
+		com_err(whoami,retval,"while changing exposure status");
 	return;
 }
 
-unset_var(argc,argv)
-	int argc;
-	char *argv[];
+void
+unset_var(int argc,
+	  char *argv[])
 {
 	int retval,i;
-	
+
 	if (argc < 2) {
 		fprintf(stderr,"Usage: %s <varname> [<varname> ... ]\n",
 			argv[0]);
@@ -389,13 +411,13 @@ unset_var(argc,argv)
 
 	for (i=1;i<argc;i++)
 		if ((retval = ZUnsetVariable(argv[i])) != ZERR_NONE)
-			ss_perror(sci_idx,retval,
+			com_err(whoami, retval,
 				  "while unsetting variable value");
 }
-	
-cancel_subs(argc,argv)
-	int argc;
-	char *argv[];
+
+void
+cancel_subs(int argc,
+	    char *argv[])
 {
 	int retval;
 	short wgport;
@@ -403,50 +425,51 @@ cancel_subs(argc,argv)
 	if (argc != 1) {
 		fprintf(stderr,"Usage: %s\n",argv[0]);
 		return;
-	} 
+	}
 
  	if ((wgport = ZGetWGPort()) == -1) {
-		ss_perror(sci_idx,errno,"while finding WindowGram port");
+		com_err(whoami, errno, "while finding WindowGram port");
 		return;
-	} 
+	}
 	if ((retval = ZCancelSubscriptions((u_short)wgport)) != ZERR_NONE)
-		ss_perror(sci_idx,retval,"while cancelling subscriptions");
+		com_err(whoami, retval, "while cancelling subscriptions");
 }
 
-subscribe(argc,argv)
-	int argc;
-	char *argv[];
+void
+subscribe(int argc,
+	  char *argv[])
 {
 	int retval;
 	short wgport;
 	ZSubscription_t sub,sub2;
-	
+
 	if (argc > 4 || argc < 3) {
 		fprintf(stderr,"Usage: %s class instance [*]\n",argv[0]);
 		return;
 	}
-	
+
 	sub.zsub_class = argv[1];
 	sub.zsub_classinst = argv[2];
 	sub.zsub_recipient = (argc == 3)?ZGetSender():argv[3];
 
 	fix_macros(&sub,&sub2,1);
-	
- 	if ((wgport = ZGetWGPort()) == -1) {
-		ss_perror(sci_idx,errno,"while finding WindowGram port");
-		return;
-	} 
 
-	retval = (*argv[0] == 's') ? ZSubscribeTo(&sub2,1,(u_short)wgport) :
+ 	if ((wgport = ZGetWGPort()) == -1) {
+		com_err(whoami, errno, "while finding WindowGram port");
+		return;
+	}
+
+	retval = (*argv[0] == 's') ?
+		ZSubscribeToSansDefaults(&sub2,1,(u_short)wgport) :
 		ZUnsubscribeTo(&sub2,1,(u_short)wgport);
-	
+
 	if (retval != ZERR_NONE)
-		ss_perror(sci_idx,retval,"while subscribing");
+		com_err(whoami, retval, "while subscribing");
 } 
 
-sub_file(argc,argv)
-	int argc;
-	char *argv[];
+void
+sub_file(int argc,
+	 char *argv[])
 {
 	ZSubscription_t sub;
 	short wgport;
@@ -457,7 +480,7 @@ sub_file(argc,argv)
 	}
 
 	if (argv[1][0] == '!') {
-		ss_perror(sci_idx,0,
+		com_err(whoami, 0,
 			  (!strcmp(argv[0],"add_unsubscription") ||
 			   !strcmp(argv[0],"add_un") ||
 			   !strcmp(argv[0],"delete_unsubscription") ||
@@ -468,15 +491,14 @@ sub_file(argc,argv)
 	}
 	sub.zsub_class = argv[1];
 	sub.zsub_classinst = argv[2];
-	sub.zsub_recipient = (argc == 3)?TOKEN_ME:TOKEN_WILD;
-
+	sub.zsub_recipient = (argc == 3)?TOKEN_ME:argv[3];
 
 	if (make_exist(subsname))
 		return;
  	if ((wgport = ZGetWGPort()) == -1) {
-		ss_perror(sci_idx,errno,"while finding WindowGram port");
+		com_err(whoami, errno, "while finding WindowGram port");
 		return;
-	} 
+	}
 
 	if (!strcmp(argv[0],"add"))
 		add_file(wgport,&sub,0);
@@ -491,14 +513,14 @@ sub_file(argc,argv)
 		 !strcmp(argv[0],"del_un")) {
 		del_file(wgport,&sub,1);
 	} else
-		ss_perror(sci_idx,0,"unknown command name");
+		com_err(whoami, 0, "unknown command name");
 	return;
 }
 
-add_file(wgport,subs,unsub)
-short wgport;
-ZSubscription_t *subs;
-int unsub;
+void
+add_file(short wgport,
+	 ZSubscription_t *subs,
+	 int unsub)
 {
 	FILE *fp;
 	char errbuf[BUFSIZ];
@@ -508,34 +530,36 @@ int unsub;
 	(void) purge_subs(subs,ALL);	/* remove copies in the subs file */
 	if (!(fp = fopen(subsname,"a"))) {
 		(void) sprintf(errbuf,"while opening %s for append",subsname);
-		ss_perror(sci_idx,errno,errbuf);
+		com_err(whoami, errno, errbuf);
 		return;
-	} 
+	}
 	fprintf(fp,"%s%s,%s,%s\n",
 		unsub ? "!" : "",
 		subs->zsub_class, subs->zsub_classinst, subs->zsub_recipient);
 	if (fclose(fp) == EOF) {
 		(void) sprintf(errbuf, "while closing %s", subsname);
-		ss_perror(sci_idx, errno, errbuf);
+		com_err(whoami, errno, errbuf);
 		return;
 	}
 	fix_macros(subs,&sub2,1);
-	if (retval = (unsub ? ZUnsubscribeTo(&sub2,1,(u_short)wgport) :
-		       ZSubscribeTo(&sub2,1,(u_short)wgport)))
-		ss_perror(sci_idx,retval,
+	retval = (unsub
+		  ? ZUnsubscribeTo(&sub2,1,(u_short)wgport)
+		  : ZSubscribeToSansDefaults(&sub2,1,(u_short)wgport));
+	if (retval)
+		com_err(whoami, retval,
 			  unsub ? "while unsubscribing" :
 			  "while subscribing");
 	return;
 }
 
-del_file(wgport,subs,unsub)
-short wgport;
-register ZSubscription_t *subs;
-int unsub;
+void
+del_file(short wgport,
+	 register ZSubscription_t *subs,
+	 int unsub)
 {
 	ZSubscription_t sub2;
 	int retval;
-	
+
 	retval = purge_subs(subs, unsub ? UNSUBONLY : SUBONLY);
 	if (retval == ERR)
 		return;
@@ -548,20 +572,19 @@ int unsub;
 	fix_macros(subs,&sub2,1);
 	if ((retval = ZUnsubscribeTo(&sub2,1,(u_short)wgport)) !=
 	    ZERR_NONE)
-		ss_perror(sci_idx,retval,"while unsubscribing");
+		com_err(whoami, retval, "while unsubscribing");
 	return;
 }
 
 int
-purge_subs(subs,which)
-register ZSubscription_t *subs;
-int which;
+purge_subs(register ZSubscription_t *subs,
+	   int which)
 {
 	FILE *fp,*fpout;
 	char errbuf[BUFSIZ],subline[BUFSIZ];
 	char backup[BUFSIZ],ourline[BUFSIZ];
 	int delflag = NOT_REMOVED;
-	int keep;
+	int keep = 0;
 
 	switch (which) {
 	case SUBONLY:
@@ -569,7 +592,7 @@ int which;
 	case ALL:
 		break;
 	default:
-		ss_perror(sci_idx,0,"internal error in purge_subs");
+		com_err(whoami, 0, "internal error in purge_subs");
 		return(ERR);
 	}
 
@@ -580,18 +603,18 @@ int which;
 
 	if (!(fp = fopen(subsname,"r"))) {
 		(void) sprintf(errbuf,"while opening %s for read",subsname);
-		ss_perror(sci_idx,errno,errbuf);
+		com_err(whoami, errno, errbuf);
 		return(ERR);
-	} 
+	}
 	(void) strcpy(backup, subsname);
 	(void) strcat(backup, ".temp");
 	(void) unlink(backup);
 	if (!(fpout = fopen(backup,"w"))) {
 		(void) sprintf(errbuf,"while opening %s for writing",backup);
-		ss_perror(sci_idx,errno,errbuf);
+		com_err(whoami, errno, errbuf);
 		(void) fclose(fp);
 		return(ERR);
-	} 
+	}
 	for (;;) {
 		if (!fgets(subline,sizeof subline,fp))
 			break;
@@ -615,7 +638,7 @@ int which;
 			if (ferror(fpout) || (fputc('\n', fpout) == EOF)) {
 				(void) sprintf(errbuf, "while writing to %s",
 					       backup);
-				ss_perror(sci_idx, errno, errbuf);
+				com_err(whoami, errno, errbuf);
 			}
 		} else
 			delflag = REMOVED;
@@ -623,26 +646,30 @@ int which;
 	(void) fclose(fp);		/* open read-only, ignore errs */
 	if (fclose(fpout) == EOF) {
 		(void) sprintf(errbuf, "while closing %s",backup);
-		ss_perror(sci_idx, errno, errbuf);
+		com_err(whoami, errno, errbuf);
 		return(ERR);
 	}
 	if (rename(backup,subsname) == -1) {
 		(void) sprintf(errbuf,"while renaming %s to %s\n",
 			       backup,subsname);
-		ss_perror(sci_idx,errno,errbuf);
+		com_err(whoami, errno, errbuf);
 		return(ERR);
 	}
 	return(delflag);
 }
 
-load_subs(argc,argv)
-	int argc;
-	char *argv[];
+void
+load_subs(int argc,
+	  char *argv[])
 {
 	ZSubscription_t subs[SUBSATONCE],subs2[SUBSATONCE],unsubs[SUBSATONCE];
+#ifdef CMU_ZCTL_PUNT
+	ZSubscription_t punts[SUBSATONCE];
+	int pind;
+#endif
 	FILE *fp;
 	int ind,unind,lineno,i,retval,type;
-	short wgport;
+	short wgport = 0;
 	char *comma,*comma2,*file,subline[BUFSIZ];
 
 	if (argc > 2) {
@@ -658,26 +685,29 @@ load_subs(argc,argv)
 		else
 			type = SUB;
 
-	if (type != LIST) 
+	if (type != LIST)
 		if ((wgport = ZGetWGPort()) == -1) {
-			ss_perror(sci_idx,errno,
+			com_err(whoami, errno,
 				  "while finding WindowGram port");
 			return;
-		} 
+		}
 
 	file = (argc == 1) ? subsname : argv[1];
-	
+
 	fp = fopen(file,"r");
 
 	if (fp == NULL) {
-		ss_perror(sci_idx,errno,
+		com_err(whoami, errno,
 			  "while loading subscription file");
 		return;
 	}
-	
-	ind = unind = 0;
+
+#ifdef CMU_ZCTL_PUNT
+	pind =
+#endif
+	  ind = unind = 0;
 	lineno = 1;
-	
+
 	for (;;lineno++) {
 		if (!fgets(subline,sizeof subline,fp))
 			break;
@@ -698,9 +728,14 @@ load_subs(argc,argv)
 		*comma = '\0';
 		*comma2 = '\0';
 		if (type == LIST) {
-			if (*subline == '!') 
+			if (*subline == '!')
 				printf("(Un-subscription) Class %s instance %s recipient %s\n",
 				       subline+1, comma+1, comma2+1);
+#ifdef CMU_ZCTL_PUNT
+			else if(*subline == '-')
+				printf("(Punted) Class %s instance %s recipient %s\n",
+				       subline+1, comma+1, comma2+1);
+#endif
 			else
 				printf("Class %s instance %s recipient %s\n",
 				       subline, comma+1, comma2+1);
@@ -726,7 +761,27 @@ load_subs(argc,argv)
 			/* XXX check malloc return */
 			(void) strcpy(unsubs[unind].zsub_recipient,comma2+1);
 			unind++;
-		} else {
+		} else
+#ifdef CMU_ZCTL_PUNT
+		if (*subline == '-') {  /* a punt */
+			if (type == UNSUB)
+				continue;
+			punts[pind].zsub_class =
+			    (char *)malloc((unsigned)(strlen(subline)+1));
+			/* XXX check malloc return */
+			(void) strcpy(punts[pind].zsub_class,subline+1);
+			punts[pind].zsub_classinst =
+			    (char *)malloc((unsigned)(strlen(comma+1)+1));
+			/* XXX check malloc return */
+			(void) strcpy(punts[pind].zsub_classinst,comma+1);
+			punts[pind].zsub_recipient =
+			    (char *)malloc((unsigned)(strlen(comma2+1)+1));
+			/* XXX check malloc return */
+			(void) strcpy(punts[pind].zsub_recipient,comma2+1);
+			pind++;
+		} else
+#endif
+		{
 			subs[ind].zsub_class =
 			    (char *)malloc((unsigned)(strlen(subline)+1));
 			/* XXX check malloc return */
@@ -741,13 +796,31 @@ load_subs(argc,argv)
 			(void) strcpy(subs[ind].zsub_recipient,comma2+1);
 			ind++;
 		}
+#ifdef CMU_ZCTL_PUNT
+		if (pind == SUBSATONCE) {
+			 fix_macros(punts,subs2,pind);
+			 if (retval = ZPunt(subs2,pind,(u_short)wgport) !=
+			     ZERR_NONE)
+			   {
+			     com_err(whoami, retval,
+				       "while punting");
+			     goto cleanup;
+			   }
+			 for (i=0;i<pind;i++) {
+				 free(punts[i].zsub_class);
+				 free(punts[i].zsub_classinst);
+				 free(punts[i].zsub_recipient);
+			 }
+			 pind = 0;
+		}
+#endif
 		if (ind == SUBSATONCE) {
 			fix_macros(subs,subs2,ind);
 			if ((retval = (type == SUB)?
 			     ZSubscribeTo(subs2,ind,(u_short)wgport):
 			     ZUnsubscribeTo(subs2,ind,(u_short)wgport)) !=
 			    ZERR_NONE) {
-				ss_perror(sci_idx,retval,(type == SUB)?
+				com_err(whoami, retval, (type == SUB) ?
 					  "while subscribing":
 					  "while unsubscribing");
 				goto cleanup;
@@ -756,13 +829,13 @@ load_subs(argc,argv)
 				free(subs[i].zsub_class);
 				free(subs[i].zsub_classinst);
 				free(subs[i].zsub_recipient);
-			} 
+			}
 			ind = 0;
 		}
 		if (unind == SUBSATONCE) {
 			fix_macros(unsubs,subs2,unind);
 			if ((retval = ZUnsubscribeTo(subs2,unind,(u_short)wgport)) != ZERR_NONE) {
-				ss_perror(sci_idx,retval,
+				com_err(whoami, retval,
 					  "while unsubscribing to un-subscriptions");
 				goto cleanup;
 			}
@@ -770,18 +843,18 @@ load_subs(argc,argv)
 				free(unsubs[i].zsub_class);
 				free(unsubs[i].zsub_classinst);
 				free(unsubs[i].zsub_recipient);
-			} 
+			}
 			unind = 0;
 		}
 	}
-	
+
 	if (type != LIST) {
 		/* even if we have no subscriptions, be sure to send
 		   an empty packet to trigger the default subscriptions */
 		fix_macros(subs,subs2,ind);
 		if ((retval = (type == SUB)?ZSubscribeTo(subs2,ind,(u_short)wgport):
 		     ZUnsubscribeTo(subs2,ind,(u_short)wgport)) != ZERR_NONE) {
-			ss_perror(sci_idx,retval,(type == SUB)?
+			com_err(whoami, retval, (type == SUB) ?
 				  "while subscribing":
 				  "while unsubscribing");
 			goto cleanup;
@@ -790,39 +863,59 @@ load_subs(argc,argv)
 			fix_macros(unsubs,subs2,unind);
 			if ((retval =
 			     ZUnsubscribeTo(subs2,unind,(u_short)wgport)) != ZERR_NONE) {
-				ss_perror(sci_idx,retval,
+				com_err(whoami, retval,
 					  "while unsubscribing to un-subscriptions");
 				goto cleanup;
 			}
 		}
+#ifdef CMU_ZCTL_PUNT
+		if (pind) {
+			fix_macros(punts,subs2,pind);
+			if (retval = ZPunt(subs2,pind,(u_short)wgport) !=
+			    ZERR_NONE)
+			{
+			  com_err(whoami,retval,
+				    "while punting");
+			  goto cleanup;
+			}
+		}
+#endif
 	}
 cleanup:
 	for (i=0;i<ind;i++) {
 	  free(subs[i].zsub_class);
 	  free(subs[i].zsub_classinst);
 	  free(subs[i].zsub_recipient);
-	} 
+	}
 	for (i=0;i<unind;i++) {
 	  free(unsubs[i].zsub_class);
 	  free(unsubs[i].zsub_classinst);
 	  free(unsubs[i].zsub_recipient);
-	} 
+	}
+#ifdef CMU_ZCTL_PUNT
+	for (i=0;i<pind;i++) {
+	  free(punts[i].zsub_class);
+	  free(punts[i].zsub_classinst);
+	  free(punts[i].zsub_recipient);
+	}
+#endif
 
 	(void) fclose(fp);	/* ignore errs--file is read-only */
 	return;
 }
 
-current(argc,argv)
-	int argc;
-	char *argv[];
+void
+current(int argc,
+	char *argv[])
 {
-	FILE *fp;
+	FILE *fp = NULL;
 	char errbuf[BUFSIZ];
 	ZSubscription_t subs;
 	int i,nsubs,retval,save,one,defs;
 	short wgport;
-	char *file,backup[BUFSIZ];
-	
+	char backup[BUFSIZ];
+	char *file = NULL;
+
 	save = 0;
 	defs = 0;
 
@@ -838,10 +931,10 @@ current(argc,argv)
 
 	if (!defs)
 		if ((wgport = ZGetWGPort()) == -1) {
-			ss_perror(sci_idx,errno,
+			com_err(whoami, errno,
 				  "while finding WindowGram port");
 			return;
-		} 
+		}
 
 	if (defs)
 		retval = ZRetrieveDefaultSubscriptions(&nsubs);
@@ -857,7 +950,7 @@ current(argc,argv)
 	}
 	else
 		if (retval != ZERR_NONE) {
-			ss_perror(sci_idx,retval,"retrieving subscriptions");
+			com_err(whoami, retval, "retrieving subscriptions");
 			return;
 		}
 
@@ -868,22 +961,22 @@ current(argc,argv)
 		if (!(fp = fopen(backup,"w"))) {
 			(void) sprintf(errbuf,"while opening %s for write",
 				       backup);
-			ss_perror(sci_idx,errno,errbuf);
+			com_err(whoami, errno, errbuf);
 			return;
 		}
 	}
-	
+
 	for (i=0;i<nsubs;i++) {
 		one = 1;
 		if ((retval = ZGetSubscriptions(&subs,&one)) != ZERR_NONE) {
-			ss_perror(sci_idx,retval,"while getting subscription");
+			com_err(whoami, retval, "while getting subscription");
 			if (save) {
 				fprintf(stderr,"Subscriptions file not modified\n");
 				(void) fclose(fp);
 				(void) unlink(backup);
 			}
 			return;
-		} 
+		}
 		if (save)
 			fprintf(fp,"%s,%s,%s\n",subs.zsub_class,
 				subs.zsub_classinst, subs.zsub_recipient);
@@ -896,44 +989,45 @@ current(argc,argv)
 	if (save) {
 		if (fclose(fp) == EOF) {
 			(void) sprintf(errbuf, "while closing %s", backup);
-			ss_perror(sci_idx, errno, errbuf);
+			com_err(whoami, errno, errbuf);
 			return;
 		}
 		if (rename(backup,file) == -1) {
 			(void) sprintf(errbuf,"while renaming %s to %s",
 				       backup,file);
-			ss_perror(sci_idx,retval,errbuf);
+			com_err(whoami, retval, errbuf);
 			(void) unlink(backup);
 		}
 	}
 }
 
-make_exist(filename)
-	char *filename;
+int
+make_exist(char *filename)
 {
 	char errbuf[BUFSIZ];
 	FILE *fpout;
-	
+
 	if (!access(filename,F_OK))
 		return (0);
 
 	if (!(fpout = fopen(filename,"w"))) {
 		(void) sprintf(errbuf,"while opening %s for write",filename);
-		ss_perror(sci_idx,errno,errbuf);
+		com_err(whoami, errno, errbuf);
 		return (1);
 	}
 
 	if (fclose(fpout) == EOF) {
 		(void) sprintf(errbuf, "while closing %s", filename);
-		ss_perror(sci_idx, errno, errbuf);
+		com_err(whoami, errno, errbuf);
 		return(1);
 	}
 	return (0);
 }
 
-fix_macros(subs,subs2,num)
-	ZSubscription_t *subs,*subs2;
-	int num;
+void
+fix_macros(ZSubscription_t *subs,
+	   ZSubscription_t *subs2,
+	   int num)
 {
 	int i;
 
@@ -945,9 +1039,8 @@ fix_macros(subs,subs2,num)
 	}
 }
 
-fix_macros2(src,dest)
-	register char *src;
-	char **dest;
+void
+fix_macros2(char *src, char **dest)
 {
 	if (!strcmp(src,TOKEN_HOSTNAME)) {
 		*dest = ourhost;
@@ -960,3 +1053,166 @@ fix_macros2(src,dest)
 	if (!strcmp(src,TOKEN_ME))
 		*dest = ZGetSender();
 }
+
+int
+do_punt(int argc, char **argv)
+{
+#ifdef CMU_ZCTL_PUNT
+  char *class, *inst, *recip, *msg, *whoami = argv[0];
+  int retval, punt;
+  short newport;
+  struct sockaddr_in newsin;
+  ZNotice_t notice;
+
+  if (! strcmp(whoami, "punt")) punt = 1;
+  else punt = 0;
+
+  switch (argc) {
+  case 2:
+    class = "message";
+    inst = argv[1];
+    recip = "";
+    break;
+  case 3:
+    class = argv[1];
+    inst = argv[2];
+    recip = "";
+    break;
+  case 4:
+    class = argv[1];
+    inst = argv[2];
+    recip = argv[3];
+    if (*argv[3] == '*') /* fixed so realm punts would work (vs0r) */
+      if (*(argv[3]+1) == '@')
+      if (!strcmp((char *)ZGetRealm(), (char *)ZExpandRealm(argv[3]+2)))
+        /*check local*/
+        recip = "";
+    break;
+  default:
+    fprintf(stderr, "Usages:\n");
+    fprintf(stderr, "\t%s instance\n", whoami);
+    fprintf(stderr, "\t%s class instance\n", whoami);
+    fprintf(stderr, "\t%s class instance recipient\n", whoami);
+    return 1;
+  }
+
+  retval = ZOpenPort((u_short *) 0);
+  if(retval != ZERR_NONE) {
+    com_err(whoami, retval, "while opening Zephyr port.");
+    return 1;
+  }
+
+  newsin = ZGetDestAddr();
+  if ((newport = ZGetWGPort()) == -1) {
+    fprintf(stderr, "%s: Can't find windowgram port\n", whoami);
+    return 1;
+  }
+
+  newsin.sin_port = (unsigned short) newport;
+  if ((retval = ZSetDestAddr(&newsin)) != ZERR_NONE) {
+    com_err(whoami,retval,"while setting destination address");
+    return 1;
+  }
+
+  msg = (char *) malloc(strlen(class) + strlen(inst) + strlen(recip) + 4);
+  sprintf(msg, "%s%c%s%c%s", class, '\0', inst, '\0', recip);
+
+  if (*recip == '*')
+    if (*(recip+1) == '@')
+      if (strcmp(recip+2, (char *)ZExpandRealm(recip+2)))
+        sprintf(msg, "%s%c%s%c%s", class, '\0', inst, '\0',
+                (char *)ZExpandRealm(recip+2));
+  printf("%s <%s,%s,%s>\n", punt ? "Punting" : "Unpunting",
+       class, inst, *recip ? recip : "*");
+
+  memset((char *) &notice, 0, sizeof(ZNotice_t));
+  notice.z_kind = UNSAFE;
+  notice.z_class = WG_CTL_CLASS;
+  notice.z_class_inst = WG_CTL_USER;
+  notice.z_recipient = "";
+  notice.z_default_format = "";
+  notice.z_opcode = (punt) ? "SUPPRESS" : "UNSUPPRESS";
+  notice.z_port = 0;
+  notice.z_message = msg;
+  notice.z_message_len = strlen(class)+strlen(inst)+strlen(recip)+3;
+
+  if ((retval = ZSendNotice(&notice,ZNOAUTH)) != ZERR_NONE)
+    fprintf(stderr,"%s: while sending notice\n",whoami);
+
+  free(msg);
+
+  ZClosePort();
+#endif
+  return 0;
+}
+
+int
+list_punts(int argc, char **argv)
+{
+#ifdef CMU_ZCTL_PUNT
+  ZNotice_t notice;
+  int retval, lensofar;
+  struct sockaddr_in old, to, from;
+  u_short ourport, zwgcport;
+  char *msg;
+
+  ourport=0;
+  retval = ZOpenPort(&ourport);
+  if(retval != ZERR_NONE) {
+    com_err("zctl", retval, "while opening Zephyr port.");
+    return 1;
+  }
+
+  old = ZGetDestAddr();
+  to = old;
+  if ((zwgcport = ZGetWGPort()) == (u_short)-1) {
+    fprintf(stderr, "zctl: Can't find windowgram port\n");
+    return 1;
+  }
+
+  to.sin_port = (u_short) zwgcport;
+  if ((retval = ZSetDestAddr(&to)) != ZERR_NONE) {
+    com_err("zctl",retval,"while setting destination address");
+    return 1;
+  }
+
+  memset((char *) &notice, 0, sizeof(ZNotice_t));
+  notice.z_kind = UNSAFE;
+  notice.z_class = WG_CTL_CLASS;
+  notice.z_class_inst = WG_CTL_USER;
+  notice.z_recipient = "";
+  notice.z_default_format = "";
+  notice.z_opcode = "LIST-SUPPRESSED" /*USER_LIST_SUPPRESSED*/;
+  notice.z_port = ourport;
+  notice.z_message = NULL;
+  notice.z_message_len = 0;
+
+  if ((retval = ZSendNotice(&notice,ZNOAUTH)) != ZERR_NONE)
+    com_err("zctl",retval,"while sending notice");
+
+  if ((retval = ZReceiveNotice(&notice,&from)) != ZERR_NONE)
+    com_err("zctl",retval,"while receiving ack");
+
+  (void) ZFreeNotice(&notice);
+
+  if ((retval = ZReceiveNotice(&notice,&from)) != ZERR_NONE)
+    com_err("zctl",retval,"while receiving notice");
+
+  notice.z_auth = ZCheckAuthentication(&notice, &from);
+
+  if ((retval = ZSetDestAddr(&old)) != ZERR_NONE) {
+    com_err("zctl",retval,"while resetting destination address");
+    return 1;
+  }
+
+  msg = (char *) malloc((notice.z_message_len+1) * sizeof(char));
+  (void) strncpy(msg,notice.z_message, notice.z_message_len);
+  msg[notice.z_message_len]=(char)0;
+  printf("%s", msg);
+  (void) free(msg);
+  (void) ZFreeNotice(&notice);
+  (void) ZClosePort();
+
+#endif /* CMU_ZCTL_PUNT */
+  return 0;
+}
