@@ -3,7 +3,7 @@
  *
  *	Created by:	Robert French
  *
- *	$Id: zwrite.c 2522 2009-08-08 03:08:30Z kcr@ATHENA.MIT.EDU $
+ *	$Id: zwrite.c 2589 2010-08-22 18:15:05Z kcr@ATHENA.MIT.EDU $
  *
  *	Copyright (c) 1987,1988 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -17,7 +17,7 @@
 #include <pwd.h>
 
 #ifndef lint
-static const char rcsid_zwrite_c[] = "$Id: zwrite.c 2522 2009-08-08 03:08:30Z kcr@ATHENA.MIT.EDU $";
+static const char rcsid_zwrite_c[] = "$Id: zwrite.c 2589 2010-08-22 18:15:05Z kcr@ATHENA.MIT.EDU $";
 #endif /* lint */
 
 #define DEFAULT_CLASS "MESSAGE"
@@ -322,6 +322,8 @@ main(int argc, char *argv[])
 		(void) strcpy(message+msgsize, bfr);
 		msgsize += l;
 	    }
+	    if (ferror(stdin)) /* drop the message */
+		exit(1);
 	    message = realloc(message, (unsigned)(msgsize+1));
 	}
 	else {	/* Use read so you can send binary messages... */
@@ -392,9 +394,7 @@ send_off(ZNotice_t *notice, int real)
 	}
 	if ((retval = ZIfNotice(&retnotice, (struct sockaddr_in *) 0,
 				ZCompareUIDPred, 
-				(char *)&notice->z_uid)) !=
-	    ZERR_NONE) {
-	    ZFreeNotice(&retnotice);
+				(char *)&notice->z_uid)) != ZERR_NONE) {
 	    (void) sprintf(bfr, "while waiting for acknowledgement for %s", 
 		    dest);
 	    com_err(whoami, retval, bfr);
