@@ -3,7 +3,7 @@
  *
  *	Created by:	Robert French
  *
- *	$Id: ZGetSubs.c 2135 2008-01-21 03:39:30Z kcr $
+ *	$Id: ZGetSubs.c 2626 2011-02-02 05:26:15Z kcr@ATHENA.MIT.EDU $
  *
  *	Copyright (c) 1987 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -11,13 +11,11 @@
  */
 
 #ifndef lint
-static const char rcsid_ZGetSubscriptions_c[] = "$Id: ZGetSubs.c 2135 2008-01-21 03:39:30Z kcr $";
+static const char rcsid_ZGetSubscriptions_c[] = "$Id: ZGetSubs.c 2626 2011-02-02 05:26:15Z kcr@ATHENA.MIT.EDU $";
 #endif
 
 #include <internal.h>
 
-#define min(a,b) ((a)<(b)?(a):(b))
-	
 Code_t
 ZGetSubscriptions(ZSubscription_t *subscription,
 		  int *numsubs)
@@ -30,7 +28,7 @@ ZGetSubscriptions(ZSubscription_t *subscription,
     if (__subscriptions_next == __subscriptions_num)
 	return (ZERR_NOMORESUBSCRIPTIONS);
 	
-    for (i=0;i<min(*numsubs, __subscriptions_num-__subscriptions_next);i++)
+    for (i = 0; i < MIN(*numsubs, __subscriptions_num-__subscriptions_next); i++)
 	subscription[i] = __subscriptions_list[i+__subscriptions_next];
 
     if (__subscriptions_num-__subscriptions_next < *numsubs)
