@@ -3,7 +3,7 @@
  *
  *	Created by:	Robert French
  *
- *	$Id: Zinternal.c 2502 2009-07-26 21:25:27Z kcr@ATHENA.MIT.EDU $
+ *	$Id: Zinternal.c 2629 2011-02-02 05:26:22Z kcr@ATHENA.MIT.EDU $
  *
  *	Copyright (c) 1987,1988,1991 by the Massachusetts Institute of
  *	Technology.
@@ -14,11 +14,10 @@
 #include <internal.h>
 #include <arpa/inet.h>
 #include <sys/socket.h>
-#include <utmp.h>
 
 #ifndef lint
 static const char rcsid_Zinternal_c[] =
-  "$Id: Zinternal.c 2502 2009-07-26 21:25:27Z kcr@ATHENA.MIT.EDU $";
+  "$Id: Zinternal.c 2629 2011-02-02 05:26:22Z kcr@ATHENA.MIT.EDU $";
 static const char copyright[] =
   "Copyright (c) 1987,1988,1991 by the Massachusetts Institute of Technology.";
 #endif
@@ -114,9 +113,9 @@ static Code_t Z_ZcodeFormatRawHeader(ZNotice_t *, char *, int, int *, char **,
 
 /* Find or insert uid in the old uids buffer.  The buffer is a sorted
  * circular queue.  We make the assumption that most packets arrive in
- * order, so we can usually search for a uid or insert it into the buffer
- * by looking back just a few entries from the end.  Since this code is
- * only executed by the client, the implementation isn't microoptimized. */
+ * order, so we can usually search for a uid or just tack it onto the end.
+ * The first entry at at buffer[start], the last is at
+ * buffer[(start + num - 1) % size] */
 static int
 find_or_insert_uid(ZUnique_Id_t *uid,
 		   ZNotice_Kind_t kind)
@@ -132,7 +131,7 @@ find_or_insert_uid(ZUnique_Id_t *uid,
 
     time_t now;
     struct _filter *new;
-    long i, j, new_size;
+    long i, new_size;
     int result;
 
     /* Initialize the uid buffer if it hasn't been done already. */
@@ -168,14 +167,10 @@ find_or_insert_uid(ZUnique_Id_t *uid,
 	result = memcmp(uid, &buffer[i % size].uid, sizeof(*uid));
 	if (result == 0 && buffer[i % size].kind == kind)
 	    return 1;
-	if (result > 0)
-	    break;
     }
 
-    /* We didn't find it; insert the uid into the buffer after i. */
-    i++;
-    for (j = start + num; j > i; j--)
-	buffer[j % size] = buffer[(j - 1) % size];
+    /* We didn't find it; stick it on the end */
+    i = start + num;
     buffer[i % size].uid = *uid;
     buffer[i % size].kind = kind;
     buffer[i % size].t = now;
@@ -338,12 +333,12 @@ Z_ReadWait(void)
 	    tmpnotice.z_message_len = 0;
 	    olddest = __HM_addr;
 	    __HM_addr = from;
-	    if ((retval = ZFormatSmallRawNotice(&tmpnotice, pkt, &len))
-		!= ZERR_NONE)
-		return(retval);
-	    if ((retval = ZSendPacket(pkt, len, 0)) != ZERR_NONE)
-		return (retval);
+	    retval = ZFormatSmallRawNotice(&tmpnotice, pkt, &len);
+	    if (retval == ZERR_NONE)
+		retval = ZSendPacket(pkt, len, 0);
 	    __HM_addr = olddest;
+	    if (retval != ZERR_NONE)
+		return retval;
 	}
 	if (find_or_insert_uid(&notice.z_uid, notice.z_kind))
 	    return(ZERR_NONE);
@@ -361,12 +356,11 @@ Z_ReadWait(void)
     if (slash) {
 	part = atoi(notice.z_multinotice);
 	partof = atoi(slash+1);
-	if (part > partof || partof == 0) {
+	if (part < 0 || part > partof || partof <= 0) {
 	    part = 0;
 	    partof = notice.z_message_len;
 	}
-    }
-    else {
+    } else {
 	part = 0;
 	partof = notice.z_message_len;
     }
@@ -529,6 +523,10 @@ Z_AddNoticeToEntry(struct _Z_InputQ *qptr,
     struct _Z_Hole *hole, *lasthole;
     struct timeval tv;
 
+    /* Make sure this notice is expirable */
+    (void) gettimeofday(&tv, (struct timezone *)0);
+    qptr->timep = tv.tv_sec;
+
     /* Bounds check. */
     if (part < 0 || notice->z_message_len < 0 || part > qptr->msg_len
 	|| notice->z_message_len > qptr->msg_len - part)
@@ -540,9 +538,6 @@ Z_AddNoticeToEntry(struct _Z_InputQ *qptr,
     else if (notice->z_checked_auth == ZAUTH_NO && qptr->auth != ZAUTH_FAILED)
 	qptr->auth = ZAUTH_NO;
 
-    (void) gettimeofday(&tv, (struct timezone *)0);
-    qptr->timep = tv.tv_sec;
-
     last = part+notice->z_message_len-1;
 
     hole = qptr->holelist;
@@ -581,15 +576,14 @@ Z_AddNoticeToEntry(struct _Z_InputQ *qptr,
 		hole = hole->next;
 	    }
 	    if (lasthole) {
-		if (!(lasthole->next = (struct _Z_Hole *)
-		      malloc(sizeof(struct _Z_InputQ))))
-		    return (ENOMEM);
+		lasthole->next = (struct _Z_Hole *)malloc(sizeof(struct _Z_Hole));
+		if (lasthole->next == NULL)
+		    return ENOMEM;
 		hole = lasthole->next;
-	    }
-	    else {
-		if (!(qptr->holelist = (struct _Z_Hole *)
-		      malloc(sizeof(struct _Z_InputQ))))
-		    return (ENOMEM);
+	    } else {
+		qptr->holelist = (struct _Z_Hole *)malloc(sizeof(struct _Z_Hole));
+		if (qptr->holelist == NULL)
+		    return ENOMEM;
 		hole = qptr->holelist;
 	    }
 	    hole->next = NULL;
@@ -605,15 +599,14 @@ Z_AddNoticeToEntry(struct _Z_InputQ *qptr,
 		hole = hole->next;
 	    }
 	    if (lasthole) {
-		if (!(lasthole->next = (struct _Z_Hole *)
-		      malloc(sizeof(struct _Z_InputQ))))
-		    return (ENOMEM);
+		lasthole->next = (struct _Z_Hole *)malloc(sizeof(struct _Z_Hole));
+		if (lasthole->next == NULL)
+		    return ENOMEM;
 		hole = lasthole->next;
-	    }
-	    else {
-		if (!(qptr->holelist = (struct _Z_Hole *)
-		      malloc(sizeof(struct _Z_InputQ))))
-		    return (ENOMEM);
+	    } else {
+		qptr->holelist = (struct _Z_Hole *)malloc(sizeof(struct _Z_Hole));
+		if (qptr->holelist == NULL)
+		    return ENOMEM;
 		hole = qptr->holelist;
 	    }
 	    hole->next = (struct _Z_Hole *) 0;
@@ -657,8 +650,6 @@ Z_FormatHeader(ZNotice_t *notice,
 {
     Code_t retval;
     static char version[BUFSIZ]; /* default init should be all \0 */
-    struct sockaddr_in name;
-    unsigned int namelen = sizeof(name);
 
     if (!notice->z_sender)
 	notice->z_sender = ZGetSender();
@@ -669,10 +660,7 @@ Z_FormatHeader(ZNotice_t *notice,
 	    if (retval != ZERR_NONE)
 		return (retval);
 	}
-	retval = getsockname(ZGetFD(), (struct sockaddr *) &name, &namelen);
-	if (retval != 0)
-	    return (retval);
-	notice->z_port = name.sin_port;
+	notice->z_port = __Zephyr_port;
     }
 
     notice->z_multinotice = "";
@@ -688,6 +676,9 @@ Z_FormatHeader(ZNotice_t *notice,
 	notice->z_sender_sockaddr.ip4.sin_family = AF_INET; /*XXX*/
 	notice->z_sender_sockaddr.ip4.sin_port = notice->z_port;
 	(void) memcpy(&notice->z_sender_sockaddr.ip4.sin_addr, &__My_addr, sizeof(__My_addr));
+#ifdef HAVE_SOCKADDR_IN_SIN_LEN
+	notice->z_sender_sockaddr.ip4.sin_len = sizeof(notice->z_sender_sockaddr.ip4);
+#endif
     }
 
     notice->z_multiuid = notice->z_uid;
@@ -709,9 +700,7 @@ Z_NewFormatHeader(ZNotice_t *notice,
 {
     Code_t retval;
     static char version[BUFSIZ]; /* default init should be all \0 */
-    struct sockaddr_in name;
     struct timeval tv;
-    unsigned int namelen = sizeof(name);
 
     if (!notice->z_sender)
 	notice->z_sender = ZGetSender();
@@ -722,10 +711,7 @@ Z_NewFormatHeader(ZNotice_t *notice,
 	    if (retval != ZERR_NONE)
 		return (retval);
 	}
-	retval = getsockname(ZGetFD(), (struct sockaddr *) &name, &namelen);
-	if (retval != 0)
-	    return (retval);
-	notice->z_port = name.sin_port;
+	notice->z_port = __Zephyr_port;
     }
 
     notice->z_multinotice = "";
@@ -740,6 +726,9 @@ Z_NewFormatHeader(ZNotice_t *notice,
     notice->z_sender_sockaddr.ip4.sin_family = AF_INET; /*XXX*/
     notice->z_sender_sockaddr.ip4.sin_port = notice->z_port;
     (void) memcpy(&notice->z_sender_sockaddr.ip4.sin_addr, &__My_addr, sizeof(__My_addr));
+#ifdef HAVE_SOCKADDR_IN_SIN_LEN
+    notice->z_sender_sockaddr.ip4.sin_len = sizeof(notice->z_sender_sockaddr.ip4);
+#endif
 
     notice->z_multiuid = notice->z_uid;
 
@@ -1251,6 +1240,9 @@ Z_SendFragmentedNotice(ZNotice_t *notice,
 	    notice->z_sender_sockaddr.ip4.sin_family = AF_INET; /*XXX*/
 	    notice->z_sender_sockaddr.ip4.sin_port = notice->z_port;
 	    (void) memcpy(&notice->z_sender_sockaddr.ip4.sin_addr, &__My_addr, sizeof(__My_addr));
+#ifdef HAVE_SOCKADDR_IN_SIN_LEN
+	    notice->z_sender_sockaddr.ip4.sin_len = sizeof(notice->z_sender_sockaddr.ip4);
+#endif
 	}
 	message_len = min(notice->z_message_len-offset, fragsize);
 	partnotice.z_message = notice->z_message+offset;
@@ -1281,13 +1273,11 @@ Code_t Z_XmitFragment(ZNotice_t *notice,
     return(ZSendPacket(buf, len, wait));
 }
 
-#ifdef Z_DEBUG
 /* For debugging printing */
 const char *const ZNoticeKinds[] = {
     "UNSAFE", "UNACKED", "ACKED", "HMACK", "HMCTL", "SERVACK", "SERVNAK",
     "CLIENTACK", "STAT"
 };
-#endif
 
 #ifdef Z_DEBUG
 
@@ -1318,7 +1308,7 @@ Z_debug_stderr(const char *format,
 
 #undef ZSetDebug
 void
-ZSetDebug(void (*proc) __P((const char *, va_list, void *)),
+ZSetDebug(void (*proc)(const char *, va_list, void *),
 	  char *arg)
 {
     __Z_debug_print = proc;
