@@ -3,9 +3,7 @@
  *
  *	Created by:	John T. Kohl
  *
- *	$Source: /mit/zephyr/repository/zephyr/server/acl.h,v $
- *	$Author: ghudson $
- *	$Header: /mit/zephyr/repository/zephyr/server/acl.h,v 3.1 1995/06/30 22:10:57 ghudson Exp $
+ *	$Id: acl.h 2527 2009-08-09 18:42:32Z kcr@ATHENA.MIT.EDU $
  *
  *	Copyright (c) 1987 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -17,11 +15,11 @@
 #ifndef	__ACL__
 #define	__ACL__
 
-int acl_add __P((char *, char *));
-int acl_check __P((char *, char *));
-int acl_delete __P((char *, char *));
-int acl_initialize __P((char *, int));
-void acl_cache_reset __P((void));
+int acl_add(char *, char *);
+int acl_check(char *, char *);
+int acl_delete(char *, char *);
+int acl_initialize(char *, int);
+void acl_cache_reset(void);
 
 #endif /* __ACL__ */
 
