@@ -3,12 +3,12 @@
  *
  *	Created by:	John T. Kohl
  *
- *	$Source: /mit/zephyr/repository/zephyr/server/main.c,v $
- *	$Author: ghudson $
+ *	$Source: /afs/dev.mit.edu/source/repository/athena/lib/zephyr/server/main.c,v $
+ *	$Author: kcr@ATHENA.MIT.EDU $
  *
  *	Copyright (c) 1987,1988,1991 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
- *	"mit-copyright.h". 
+ *	"mit-copyright.h".
  */
 
 #include <zephyr/mit-copyright.h>
@@ -19,7 +19,7 @@
 #ifndef lint
 #ifndef SABER
 static const char rcsid_main_c[] =
-    "$Id: main.c,v 1.66 1995/07/21 19:27:01 ghudson Exp $";
+    "$Id: main.c 2621 2010-12-19 20:03:08Z kcr@ATHENA.MIT.EDU $";
 #endif
 #endif
 
@@ -50,39 +50,35 @@ static const char rcsid_main_c[] =
   (if the client has not acknowledged a packet after a given timeout).
 */
 
-#define	EVER		(;;)		/* don't stop looping */
-
-static int do_net_setup __P((void));
-static int initialize __P((void));
-static void usage __P((void));
-static void do_reset __P((void));
-static RETSIGTYPE bye __P((int));
-static RETSIGTYPE dbug_on __P((int));
-static RETSIGTYPE dbug_off __P((int));
-static RETSIGTYPE sig_dump_db __P((int));
-static RETSIGTYPE sig_dump_strings __P((int));
-static RETSIGTYPE reset __P((int));
-static RETSIGTYPE reap __P((int));
-static void read_from_dump __P((char *dumpfile));
-static void dump_db __P((void));
-static void dump_strings __P((void));
+static int do_net_setup(void);
+static int initialize(void);
+static void usage(void);
+static void do_reset(void);
+static RETSIGTYPE bye(int);
+static RETSIGTYPE dbug_on(int);
+static RETSIGTYPE dbug_off(int);
+static RETSIGTYPE sig_dump_db(int);
+static RETSIGTYPE reset(int);
+static RETSIGTYPE reap(int);
+static void read_from_dump(char *dumpfile);
+static void dump_db(void);
+static void dump_strings(void);
 
 #ifndef DEBUG
-static void detach __P((void));
+static void detach(void);
 #endif
 
 static short doreset = 0;		/* if it becomes 1, perform
 					   reset functions */
 
+int nfds;				/* max file descriptor for select() */
 int srv_socket;				/* dgram socket for clients
 					   and other servers */
 int bdump_socket = -1;			/* brain dump socket fd
 					   (closed most of the time) */
 fd_set interesting;			/* the file descrips we are listening
 					   to right now */
-int nfildes;				/* number to look at in select() */
 struct sockaddr_in srv_addr;		/* address of the socket */
-struct timeval nexthost_tv;		/* time till next timeout for select */
 
 Unacked *nacklist = NULL;		/* list of packets waiting for ack's */
 
@@ -90,23 +86,24 @@ unsigned short hm_port;			/* host manager receiver port */
 unsigned short hm_srv_port;		/* host manager server sending port */
 
 char *programname;			/* set to the basename of argv[0] */
-char myname[MAXHOSTNAMELEN];		/* my host name */
+char myname[NS_MAXDNAME];		/* my host name */
 
-#ifndef ZEPHYR_USES_HESIOD
 char list_file[128];
+#ifdef HAVE_KRB5
+char keytab_file[128];
+static char tkt5_file[256];
 #endif
-#ifdef ZEPHYR_USES_KERBEROS
+#ifdef HAVE_KRB4
 char srvtab_file[128];
 static char tkt_file[128];
 #endif
+#if defined(HAVE_KRB4) || defined(HAVE_KRB5)
+char my_realm[REALM_SZ];
+#endif
 char acl_dir[128];
 char subs_file[128];
 
 int zdebug;
-#ifdef DEBUG_MALLOC
-int dump_malloc_stats = 0;
-unsigned long m_size;
-#endif
 #ifdef DEBUG
 int zalone;
 #endif
@@ -117,19 +114,37 @@ static int dump_db_flag = 0;
 static int dump_strings_flag = 0;
 
 u_long npackets;			/* number of packets processed */
-long uptime;				/* when we started operations */
+time_t uptime;				/* when we started operations */
 static int nofork;
 struct in_addr my_addr;
 char *bdump_version = "1.2";
 
+#ifdef HAVE_KRB5
+int bdump_auth_proto = 5;
+#else /* HAVE_KRB5 */
+#ifdef HAVE_KRB4
+int bdump_auth_proto = 4;
+#else /* HAVE_KRB4 */
+int bdump_auth_proto = 0;
+#endif /* HAVE_KRB4 */
+#endif /* HAVE_KRB5 */
+
+#ifdef HAVE_KRB5
+krb5_ccache Z_krb5_ccache;
+krb5_keyblock *__Zephyr_keyblock;
+#else
+#ifdef HAVE_KRB4
+C_Block __Zephyr_session;
+#endif
+#endif
+
 int
-main(argc, argv)
-    int argc;
-    char **argv;
+main(int argc,
+     char **argv)
 {
     int nfound;			/* #fildes ready on select */
     fd_set readable;
-    struct timeval *tvp;
+    struct timeval tv;
     int init_from_dump = 0;
     char *dumpfile;
 #ifdef _POSIX_VERSION
@@ -139,22 +154,24 @@ main(argc, argv)
     extern char *optarg;
     extern int optind;
 
-#ifndef ZEPHYR_USES_HESIOD
-    sprintf(list_file, "%s/%s", CONFDIR, SERVER_LIST_FILE);
+    sprintf(list_file, "%s/zephyr/%s", SYSCONFDIR, SERVER_LIST_FILE);
+#ifdef HAVE_KRB4
+    sprintf(srvtab_file, "%s/zephyr/%s", SYSCONFDIR, ZEPHYR_SRVTAB);
+    strcpy(tkt_file, ZEPHYR_TKFILE);
 #endif
-#ifdef ZEPHYR_USES_KERBEROS
-    sprintf(srvtab_file, "%s/%s", CONFDIR, ZEPHYR_SRVTAB);
-    sprintf(tkt_file, "%s/%s", CONFDIR, ZEPHYR_TKFILE);
+#ifdef HAVE_KRB5
+    sprintf(keytab_file, "%s/zephyr/%s", SYSCONFDIR, ZEPHYR_KEYTAB);
+    strcpy(tkt5_file, ZEPHYR_TK5FILE);
 #endif
-    sprintf(acl_dir, "%s/%s", CONFDIR, ZEPHYR_ACL_DIR);
-    sprintf(subs_file, "%s/%s", CONFDIR, DEFAULT_SUBS_FILE);
+    sprintf(acl_dir, "%s/zephyr/%s", SYSCONFDIR, ZEPHYR_ACL_DIR);
+    sprintf(subs_file, "%s/zephyr/%s", SYSCONFDIR, DEFAULT_SUBS_FILE);
 
     /* set name */
     programname = strrchr(argv[0],'/');
     programname = (programname) ? programname + 1 : argv[0];
 
     /* process arguments */
-    while ((optchar = getopt(argc, argv, "dsnv:f:")) != EOF) {
+    while ((optchar = getopt(argc, argv, "dsnv4f:k:")) != EOF) {
 	switch(optchar) {
 	  case 'd':
 	    zdebug = 1;
@@ -167,6 +184,11 @@ main(argc, argv)
 	  case 'n':
 	    nofork = 1;
 	    break;
+	  case 'k':
+#if defined(HAVE_KRB4) || defined(HAVE_KRB5)
+	    strncpy(my_realm, optarg, REALM_SZ);
+#endif
+	    break;
 	  case 'v':
 	    bdump_version = optarg;
 	    break;
@@ -174,6 +196,9 @@ main(argc, argv)
 	    init_from_dump = 0;
 	    dumpfile = optarg;
 	    break;
+	case '4':
+	    bdump_auth_proto = 4;
+	    break;
 	  case '?':
 	  default:
 	    usage();
@@ -181,12 +206,12 @@ main(argc, argv)
 	}
     }
 
-#ifdef ZEPHYR_USES_KERBEROS
+#ifdef HAVE_KRB4
     /* if there is no readable srvtab and we are not standalone, there
        is no possible way we can succeed, so we exit */
 
     if (access(srvtab_file, R_OK)
-#ifdef DEBUG		
+#ifdef DEBUG
 	&& !zalone
 #endif /* DEBUG */
 	) {
@@ -194,7 +219,14 @@ main(argc, argv)
 		srvtab_file);
 	exit(1);
     }
-#endif /* ZEPHYR_USES_KERBEROS */
+    /* Use local realm if not specified on command line. */
+    if (!*my_realm) {
+	if (krb_get_lrealm(my_realm, 1) != KSUCCESS) {
+	    fputs("Couldn't get local Kerberos realm; exiting.\n", stderr);
+	    exit(1);
+	}
+    }
+#endif /* HAVE_KRB4 */
 
 #ifndef DEBUG
     if (!nofork)
@@ -208,12 +240,10 @@ main(argc, argv)
     if (zalone)
 	syslog(LOG_DEBUG, "standalone operation");
 #endif
-#if 0
     if (zdebug)
 	syslog(LOG_DEBUG, "debugging on");
-#endif
 
-    /* set up sockets & my_addr and myname, 
+    /* set up sockets & my_addr and myname,
        find other servers and set up server table, initialize queues
        for retransmits, initialize error tables,
        set up restricted classes */
@@ -237,7 +267,7 @@ main(argc, argv)
     FD_ZERO(&interesting);
     FD_SET(srv_socket, &interesting);
 
-    nfildes = srv_socket + 1;
+    nfds = srv_socket + 1;
 
 
 #ifdef _POSIX_VERSION
@@ -260,11 +290,6 @@ main(argc, argv)
     action.sa_handler = sig_dump_db;
     sigaction(SIGFPE, &action, NULL);
 
-#ifdef SIGEMT
-    action.sa_handler = sig_dump_strings;
-    sigaction(SIGEMT, &action, NULL);
-#endif
-
     action.sa_handler = reset;
     sigaction(SIGHUP, &action, NULL);
 #else /* !posix */
@@ -274,9 +299,6 @@ main(argc, argv)
     signal(SIGUSR2, dbug_off);
     signal(SIGCHLD, reap);
     signal(SIGFPE, sig_dump_db);
-#ifdef SIGEMT
-    signal(SIGEMT, sig_dump_strings);
-#endif
     signal(SIGHUP, reset);
 #endif /* _POSIX_VERSION */
 
@@ -285,14 +307,10 @@ main(argc, argv)
     /* Reinitialize t_local now that initialization is done. */
     gettimeofday(&t_local, NULL);
     uptime = NOW;
-#ifdef ZEPHYR_USES_KERBEROS
-    timer_set_rel(SWEEP_INTERVAL, sweep_ticket_hash_table, NULL);
-#endif
 
-#ifdef DEBUG_MALLOC
-    malloc_inuse(&m_size);
-#endif
-    for EVER {
+    realm_wakeup();
+
+    for (;;) {
 	if (doreset)
 	    do_reset();
 
@@ -301,24 +319,8 @@ main(argc, argv)
 	if (dump_strings_flag)
 	    dump_strings();
 
-	nexthost_tv.tv_usec = 0;
-	tvp = &nexthost_tv;
+	timer_process();
 
-	if (nexttimo != 0L) {
-	    nexthost_tv.tv_sec = nexttimo - NOW;
-	    if (nexthost_tv.tv_sec <= 0) {
-		/* timeout has passed! */
-		/* so we process one timeout, then pop to
-		   select, polling for input.  This way we get
-		   work done even if swamped with many
-		   timeouts */
-		/* this will reset nexttimo */
-		timer_process();
-		nexthost_tv.tv_sec = 0;
-	    }
-	} else {			/* no timeouts to process */
-	    nexthost_tv.tv_sec = 15;
-	}
 	readable = interesting;
 	if (msgs_queued()) {
 	    /* when there is input in the queue, we
@@ -326,27 +328,17 @@ main(argc, argv)
 	    nfound = 1;
 	    FD_ZERO(&readable);
 	} else  {
-	    nfound = select(nfildes, &readable, NULL, NULL, tvp);
+	    nfound = select(nfds, &readable, NULL, NULL, timer_timeout(&tv));
 	}
 
 	/* Initialize t_local for other uses */
 	gettimeofday(&t_local, (struct timezone *)0);
-		
+
 	/* don't flame about EINTR, since a SIGUSR1 or SIGUSR2
 	   can generate it by interrupting the select */
 	if (nfound < 0) {
 	    if (errno != EINTR)
 		syslog(LOG_WARNING, "select error: %m");
-#ifdef DEBUG_MALLOC
-	    if (dump_malloc_stats) {
-		unsigned long foo,histid2;
-
-		dump_malloc_stats = 0;
-		foo = malloc_inuse(&histid2);
-		printf("Total inuse: %d\n",foo);
-		malloc_list(2,m_size,histid2);
-	    }
-#endif
 	    continue;
 	}
 
@@ -374,21 +366,47 @@ main(argc, argv)
    */
 
 static int
-initialize()
+initialize(void)
 {
     if (do_net_setup())
 	return(1);
 
     server_init();
 
-#ifdef ZEPHYR_USES_KERBEROS
+#ifdef HAVE_KRB4
     krb_set_tkt_string(tkt_file);
 #endif
-
-    ZInitialize();		/* set up the library */
-    init_zsrv_err_tbl();	/* set up err table */
+    realm_init();
 
     ZSetServerState(1);
+    ZInitialize();		/* set up the library */
+#ifdef HAVE_KRB5
+    krb5_cc_resolve(Z_krb5_ctx, tkt5_file, &Z_krb5_ccache);
+#ifdef HAVE_KRB5_CC_SET_DEFAULT_NAME
+    krb5_cc_set_default_name(Z_krb5_ctx, tkt5_file);
+#else
+    {
+	/* Hack to make krb5_cc_default do something reasonable */
+	char *env=(char *)malloc(strlen(tkt5_file)+12);
+	if (!env) return(1);
+	sprintf(env, "KRB5CCNAME=%s", tkt5_file);
+	putenv(env);
+    }
+#endif
+#endif
+#if defined(HAVE_KRB4) || defined(HAVE_KRB5)
+    /* Override what Zinitialize set for ZGetRealm() */
+    if (*my_realm)
+      strcpy(__Zephyr_realm, my_realm);
+#endif
+
+    /* set up err table */
+#if defined(__APPLE__) && defined(__MACH__)
+    add_error_table(&et_zsrv_error_table);
+#else
+    init_zsrv_err_tbl();
+#endif
+
     ZSetFD(srv_socket);		/* set up the socket as the input fildes */
 
     /* set up default strings */
@@ -406,43 +424,44 @@ initialize()
     return 0;
 }
 
-/* 
+/*
  * Set up the server and client sockets, and initialize my_addr and myname
  */
 
 static int
-do_net_setup()
+do_net_setup(void)
 {
     struct servent *sp;
     struct hostent *hp;
-    char hostname[MAXHOSTNAMELEN+1];
+    char hostname[NS_MAXDNAME];
     int flags;
 
-    if (gethostname(hostname, MAXHOSTNAMELEN + 1)) {
+    if (gethostname(hostname, sizeof(hostname))) {
 	syslog(LOG_ERR, "no hostname: %m");
 	return 1;
     }
     hp = gethostbyname(hostname);
-    if (!hp) {
+    if (!hp || hp->h_addrtype != AF_INET) {
 	syslog(LOG_ERR, "no gethostbyname repsonse");
-	strncpy(myname, hostname, MAXHOSTNAMELEN);
+	strncpy(myname, hostname, sizeof(myname));
 	return 1;
     }
-    strncpy(myname, hp->h_name, MAXHOSTNAMELEN);
-    memcpy(&my_addr, hp->h_addr, sizeof(hp->h_addr));
+    strncpy(myname, hp->h_name, sizeof(myname));
+    memcpy(&my_addr, hp->h_addr_list[0], hp->h_length);
 
     setservent(1);		/* keep file/connection open */
 
     memset(&srv_addr, 0, sizeof(srv_addr));
+    srv_addr.sin_family = AF_INET;
     sp = getservbyname(SERVER_SVCNAME, "udp");
     srv_addr.sin_port = (sp) ? sp->s_port : SERVER_SVC_FALLBACK;
 
     sp = getservbyname(HM_SVCNAME, "udp");
     hm_port = (sp) ? sp->s_port : HM_SVC_FALLBACK;
-	
+
     sp = getservbyname(HM_SRV_SVCNAME, "udp");
     hm_srv_port = (sp) ? sp->s_port : HM_SRV_SVC_FALLBACK;
-	
+
     srv_socket = socket(AF_INET, SOCK_DGRAM, 0);
     if (srv_socket < 0) {
 	syslog(LOG_ERR, "client_sock failed: %m");
@@ -465,7 +484,7 @@ do_net_setup()
 #endif
 
     return 0;
-}    
+}
 
 
 /*
@@ -473,18 +492,20 @@ do_net_setup()
  */
 
 static void
-usage()
+usage(void)
 {
 #ifdef DEBUG
-	fprintf(stderr,"Usage: %s [-d] [-s] [-n] [-f dumpfile]\n",programname);
+	fprintf(stderr, "Usage: %s [-d] [-s] [-n] [-k realm] [-f dumpfile]\n",
+		programname);
 #else
-	fprintf(stderr,"Usage: %s [-d] [-n] [-f dumpfile]\n",programname);
+	fprintf(stderr, "Usage: %s [-d] [-n] [-k realm] [-f dumpfile]\n",
+		programname);
 #endif /* DEBUG */
 	exit(2);
 }
 
 int
-packets_waiting()
+packets_waiting(void)
 {
     fd_set readable, initial;
     struct timeval tv;
@@ -499,11 +520,15 @@ packets_waiting()
 }
 
 static RETSIGTYPE
-bye(sig)
-    int sig;
+bye(int sig)
 {
     server_shutdown();		/* tell other servers */
-#ifdef ZEPHYR_USES_KERBEROS
+#ifdef REALM_MGMT
+    realm_shutdown();		/* tell other realms */
+#endif
+    hostm_shutdown();		/* tell our hosts */
+    kill_realm_pids();
+#ifdef HAVE_KRB4
     dest_tkt();
 #endif
     syslog(LOG_NOTICE, "goodbye (sig %d)", sig);
@@ -511,37 +536,22 @@ bye(sig)
 }
 
 static RETSIGTYPE
-dbug_on(sig)
-    int sig;
+dbug_on(int sig)
 {
     syslog(LOG_DEBUG, "debugging turned on");
-#ifdef DEBUG_MALLOC
-    dump_malloc_stats = 1;
-#endif
     zdebug = 1;
 }
 
 static RETSIGTYPE
-dbug_off(sig)
-    int sig;
+dbug_off(int sig)
 {
     syslog(LOG_DEBUG, "debugging turned off");
-#ifdef DEBUG_MALLOC
-    malloc_inuse(&m_size);
-#endif
     zdebug = 0;
 }
 
 int fork_for_dump = 0;
 
-static RETSIGTYPE
-sig_dump_strings(sig)
-    int sig;
-{
-    dump_strings_flag = 1;
-}
-
-static void dump_strings()
+static void dump_strings(void)
 {
     char filename[128];
 
@@ -568,13 +578,13 @@ static void dump_strings()
 }
 
 static RETSIGTYPE
-sig_dump_db(sig)
-    int sig;
+sig_dump_db(int sig)
 {
     dump_db_flag = 1;
 }
 
-static void dump_db()
+static void
+dump_db(void)
 {
     /* dump the in-core database to human-readable form on disk */
     FILE *fp;
@@ -600,6 +610,7 @@ static void dump_db()
     uloc_dump_locs(fp);
     client_dump_clients(fp);
     triplet_dump_subs(fp);
+    realm_dump_realms(fp);
     syslog(LOG_INFO, "dump done");
     if (fclose(fp) == EOF)
 	syslog(LOG_ERR, "can't close dump db");
@@ -610,34 +621,55 @@ static void dump_db()
 }
 
 static RETSIGTYPE
-reset(sig)
-    int sig;
+reset(int sig)
 {
-#if 1
     zdbug((LOG_DEBUG,"reset()"));
-#endif
     doreset = 1;
 }
 
 static RETSIGTYPE
-reap(sig)
-    int sig;
+reap(int sig)
 {
+    int pid, i = 0;
     int oerrno = errno;
-
+    ZRealm *rlm;
 #ifdef _POSIX_VERSION
     int waitb;
-    while (waitpid(-1, &waitb, WNOHANG) == 0) ;
 #else
     union wait waitb;
-    while (wait3 (&waitb, WNOHANG, (struct rusage*) 0) == 0) ;
+#endif
+
+    zdbug((LOG_DEBUG,"reap()"));
+#ifdef _POSIX_VERSION
+    while ((pid = waitpid(-1, &waitb, WNOHANG)) == 0)
+      { i++; if (i > 10) break; }
+#else
+    while ((pid = wait3 (&waitb, WNOHANG, (struct rusage*) 0)) == 0)
+      { i++; if (i > 10) break; }
 #endif
 
     errno = oerrno;
+
+    if (pid) {
+      if (WIFSIGNALED(waitb) == 0) {
+	if (WIFEXITED(waitb) != 0) {
+	  rlm = realm_get_realm_by_pid(pid);
+	  if (rlm) {
+	    rlm->child_pid = 0;
+	    rlm->have_tkt = 1;
+	  }
+	}
+      } else {
+	rlm = realm_get_realm_by_pid(pid);
+	if (rlm) {
+	  rlm->child_pid = 0;
+	}
+      }
+    }
 }
 
 static void
-do_reset()
+do_reset(void)
 {
     int oerrno = errno;
 #ifdef _POSIX_VERSION
@@ -645,9 +677,6 @@ do_reset()
 #else
     int omask;
 #endif
-#if 0
-    zdbug((LOG_DEBUG,"do_reset()"));
-#endif
 #ifdef _POSIX_VERSION
     sigemptyset(&mask);
     sigaddset(&mask, SIGHUP);
@@ -676,7 +705,7 @@ do_reset()
  */
 
 static void
-detach()
+detach(void)
 {
     /* detach from terminal and fork. */
     int i;
@@ -710,8 +739,7 @@ detach()
 #endif /* not DEBUG */
 
 static void
-read_from_dump(dumpfile)
-    char *dumpfile;
+read_from_dump(char *dumpfile)
 {
     /* Not yet implemented. */
     return;
