@@ -17,7 +17,7 @@
 
 #ifndef lint
 #ifndef SABER
-static const char rcsid_server_c[] = "$Id: server.c 2527 2009-08-09 18:42:32Z kcr@ATHENA.MIT.EDU $";
+static const char rcsid_server_c[] = "$Id: server.c 2630 2011-02-02 05:26:26Z kcr@ATHENA.MIT.EDU $";
 #endif
 #endif
 
@@ -87,7 +87,7 @@ static void server_queue(Server *, int, void *, int,
 static void server_hello(Server *, int);
 static void setup_server(Server *, struct in_addr *);
 static void srv_rexmit(void *);
-static void server_forw_reliable(Server *, caddr_t, int, ZNotice_t *);
+static void server_forw_reliable(Server *, void *, int, ZNotice_t *);
 static Code_t admin_dispatch(ZNotice_t *, int, struct sockaddr_in *,
 				  Server *);
 static Code_t kill_clt(ZNotice_t *, Server *);
@@ -538,7 +538,7 @@ server_kill_clt(Client *client)
     char buf[512], *lyst[2];
     ZNotice_t notice;
     ZNotice_t *pnotice; /* speed hack */
-    caddr_t pack;
+    char *pack;
     int packlen, auth;
     Code_t retval;
 
@@ -893,7 +893,7 @@ static char **
 get_server_list(char *file)
 {
     FILE *fp;
-    char buf[MAXHOSTNAMELEN];
+    char buf[NS_MAXDNAME];
     char **ret_list;
     int nused = 0;
     char *newline;
@@ -907,10 +907,10 @@ get_server_list(char *file)
     if (!ret_list)
     	return NULL;
 
-    while (fgets(buf, MAXHOSTNAMELEN, fp)) {
+    while (fgets(buf, sizeof(buf), fp)) {
 	/* nuke the newline, being careful not to overrun
 	   the buffer searching for it with strlen() */
-	buf[MAXHOSTNAMELEN - 1] = '\0';
+	buf[sizeof(buf) - 1] = '\0';
 	newline = strchr(buf, '\n');
 	if (newline)
 	    *newline = '\0';
@@ -935,7 +935,7 @@ get_server_list(char *file)
 static char **
 get_single_server(void)
 {
-    char buf[MAXHOSTNAMELEN];
+    char buf[NS_MAXDNAME];
     char **ret_list;
     int nused = 0;
     nhosts = 2;
@@ -1220,7 +1220,7 @@ server_forward(ZNotice_t *notice,
 	       struct sockaddr_in *who)
 {
     int i;
-    caddr_t pack;
+    void *pack;
     int packlen;
     Code_t retval;
 
@@ -1243,6 +1243,7 @@ server_forward(ZNotice_t *notice,
 	retval = ZNewFormatSmallRawNotice(notice, pack, &packlen);
 	if (retval != ZERR_NONE) {
 	    syslog(LOG_WARNING, "srv_fwd format: %s", error_message(retval));
+	    free(pack);
 	    continue;
 	}
 	if (otherservers[i].dumping) {
@@ -1255,7 +1256,7 @@ server_forward(ZNotice_t *notice,
 
 static void
 server_forw_reliable(Server *server,
-		     caddr_t pack,
+		     void *pack,
 		     int packlen,
 		     ZNotice_t *notice)
 {
