@@ -4,7 +4,7 @@
  *      Created by:     John T. Kohl
  *      Derived from timer_manager_ by Ken Raeburn
  *
- *      $Id: timer.c 2144 2008-01-21 07:57:32Z kcr $
+ *      $Id: timer.c 2630 2011-02-02 05:26:26Z kcr@ATHENA.MIT.EDU $
  *
  */
 
@@ -13,7 +13,7 @@
 #ifndef SABER
 #ifndef lint
 static const char rcsid[] =
-"$Id: timer.c 2144 2008-01-21 07:57:32Z kcr $";
+"$Id: timer.c 2630 2011-02-02 05:26:26Z kcr@ATHENA.MIT.EDU $";
 #endif /* lint */
 #endif /* SABER */
 
@@ -49,11 +49,11 @@ static const char rcsid[] =
  * Timer *timer_set_rel (time_rel, proc, arg)
  *      long time_rel;
  *      void (*proc)();
- *      caddr_t arg;
+ *      void *arg;
  * Timer *timer_set_abs (time_abs, proc, arg)
  *      long time_abs;
  *      void (*proc)();
- *      caddr_t arg;
+ *      void *arg;
  *
  * void timer_reset(tmr)
  *      Timer *tmr;
