@@ -3,12 +3,11 @@
  *
  *	Created by:	John T. Kohl
  *
- *	$Source: /mit/zephyr/repository/zephyr/server/uloc.c,v $
- *	$Author: ghudson $
+ *	$Id: uloc.c 2527 2009-08-09 18:42:32Z kcr@ATHENA.MIT.EDU $
  *
  *	Copyright (c) 1987,1988 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
- *	"mit-copyright.h". 
+ *	"mit-copyright.h".
  */
 
 #include <zephyr/mit-copyright.h>
@@ -18,7 +17,7 @@
 #ifndef lint
 #ifndef SABER
 static const char rcsid_uloc_c[] =
-"$Id: uloc.c,v 1.55 1995/07/08 02:49:01 ghudson Exp $";
+"$Id: uloc.c 2527 2009-08-09 18:42:32Z kcr@ATHENA.MIT.EDU $";
 #endif /* SABER */
 #endif /* lint */
 
@@ -59,7 +58,7 @@ static const char rcsid_uloc_c[] =
 
 /* WARNING: make sure this is the same as the number of strings you */
 /* plan to hand back to the user in response to a locate request, */
-/* else you will lose.  See ulogin_locate() and uloc_send_locations() */  
+/* else you will lose.  See ulogin_locate() and uloc_send_locations() */
 #define	NUM_FIELDS	3
 
 typedef enum _Exposure_type {
@@ -84,25 +83,27 @@ typedef struct _Location {
 #define QUIET		-1
 #define UNAUTH		-2
 
-static void ulogin_locate __P((ZNotice_t *notice, struct sockaddr_in *who,
-			       int auth)),
-ulogin_flush_user __P((ZNotice_t *notice));
-static Location *ulogin_find __P((ZNotice_t *notice, int strict));
-static int ulogin_setup __P((ZNotice_t *notice, Location *locs,
-			     Exposure_type exposure, struct sockaddr_in *who)),
-ulogin_add_user __P((ZNotice_t *notice, Exposure_type exposure,
-		     struct sockaddr_in *who)),
-ulogin_parse __P((ZNotice_t *notice, Location *locs)),
-ulogin_expose_user __P((ZNotice_t *notice, Exposure_type exposure));
-static Exposure_type ulogin_remove_user __P((ZNotice_t *notice, int auth,
-					     struct sockaddr_in *who,
-					     int *err_return));
-static void login_sendit __P((ZNotice_t *notice, int auth, struct sockaddr_in *who));
-static char **ulogin_marshal_locs __P((ZNotice_t *notice, int *found, int auth));
+static void ulogin_locate(ZNotice_t *notice, struct sockaddr_in *who,
+			  int auth);
+static void ulogin_flush_user(ZNotice_t *notice);
+static Location *ulogin_find(char *user, struct in_addr *host,
+			     unsigned int port);
+static Location *ulogin_find_user(char *user);
+static int ulogin_setup(ZNotice_t *notice, Location *locs,
+			Exposure_type exposure, struct sockaddr_in *who);
+static int ulogin_add_user(ZNotice_t *notice, Exposure_type exposure,
+			   struct sockaddr_in *who);
+static int ulogin_parse(ZNotice_t *notice, Location *locs);
+static Exposure_type ulogin_remove_user(ZNotice_t *notice,
+					struct sockaddr_in *who,
+					int *err_return);
+static void login_sendit(ZNotice_t *notice, int auth,
+			 struct sockaddr_in *who, int external);
+static char **ulogin_marshal_locs(ZNotice_t *notice, int *found, int auth);
 
-static int ul_equiv __P((Location *l1, Location *l2));
-
-static void free_loc __P((Location *loc));
+static void free_loc(Location *loc);
+static void ulogin_locate_forward(ZNotice_t *notice, struct sockaddr_in *who,
+				  ZRealm *realm);
 
 static Location *locations = NULL; /* ptr to first in array */
 static int num_locs = 0;	/* number in array */
@@ -112,35 +113,19 @@ static int num_locs = 0;	/* number in array */
  */
 
 Code_t
-ulogin_dispatch(notice, auth, who, server)
-    ZNotice_t *notice;
-    int auth;
-    struct sockaddr_in *who;
-    Server *server;
+ulogin_dispatch(ZNotice_t *notice,
+		int auth,
+		struct sockaddr_in *who,
+		Server *server)
 {
     Exposure_type retval;
     int err_ret;
 
-#if 0
-    zdbug((LOG_DEBUG,
-	   "ulogin_dispatch: opc=%s from=%s/%d auth=%d who=%s/%d",
-	   notice->z_opcode, notice->z_sender, ntohs (notice->z_port),
-	   auth, inet_ntoa (who->sin_addr), ntohs (who->sin_port)));
-#endif
-
     if (strcmp(notice->z_opcode, LOGIN_USER_LOGOUT) == 0) {
-#if 0
-	zdbug((LOG_DEBUG,"logout"));
-#endif
-	retval = ulogin_remove_user(notice, auth, who, &err_ret);
+	retval = ulogin_remove_user(notice, who, &err_ret);
 	switch (retval) {
 	  case NONE:
 	    if (err_ret == UNAUTH) {
-#if 0
-		zdbug((LOG_DEBUG, "unauth logout: %s %d",
-		       inet_ntoa(who->sin_addr),
-		       ntohs(notice->z_port)));
-#endif
 		if (server == me_server)
 		    clt_ack(notice, who, AUTH_FAILED);
 		return ZERR_NONE;
@@ -148,7 +133,7 @@ ulogin_dispatch(notice, auth, who, server)
 		if (server == me_server)
 		    clt_ack(notice, who, NOT_FOUND);
 		return ZERR_NONE;
-	    } 
+	    }
 	    syslog(LOG_ERR,"bogus location exposure NONE, %s",
 		   notice->z_sender);
 	    break;
@@ -160,6 +145,9 @@ ulogin_dispatch(notice, auth, who, server)
 	    break;
 	  case REALM_ANN:
 	  case NET_VIS:
+	    if (server == me_server)
+		sendit(notice, 1, who, 0);
+	    break;
 	  case NET_ANN:
 	    /* currently no distinction between these.
 	       just announce */
@@ -168,7 +156,7 @@ ulogin_dispatch(notice, auth, who, server)
 	       authentic.  ulogin_remove_user checks the
 	       ip addrs */
 	    if (server == me_server)
-		sendit(notice, 1, who);
+		sendit(notice, 1, who, 1);
 	    break;
 	  default:
 	    syslog(LOG_ERR,"bogus location exposure %d/%s",
@@ -179,33 +167,21 @@ ulogin_dispatch(notice, auth, who, server)
 	    server_forward(notice, auth, who);
 	return ZERR_NONE;
     }
-    if (!bdumping && 
+    if (!bdumping &&
 	(!auth || strcmp(notice->z_sender, notice->z_class_inst) != 0))  {
-#if 1
 	zdbug((LOG_DEBUG,"unauthentic ulogin: %d %s %s", auth,
 	       notice->z_sender, notice->z_class_inst));
-#endif
 	if (server == me_server)
 	    clt_ack(notice, who, AUTH_FAILED);
 	return ZERR_NONE;
     }
     if (strcmp(notice->z_opcode, LOGIN_USER_FLUSH) == 0) {
-#if 0
-	zdbug((LOG_DEBUG, "user flush"));
-#endif
 	ulogin_flush_user(notice);
 	if (server == me_server)
 	    ack(notice, who);
     } else if (strcmp(notice->z_opcode, EXPOSE_NONE) == 0) {
-#if 0
-	zdbug((LOG_DEBUG,"no expose"));
-#endif
-	ulogin_remove_user(notice, auth, who, &err_ret);
+	ulogin_remove_user(notice, who, &err_ret);
 	if (err_ret == UNAUTH) {
-#if 0
-	    zdbug((LOG_DEBUG, "unauth noexpose: %s/%d",
-		   inet_ntoa(who->sin_addr), ntohs(notice->z_port)));
-#endif
 	    if (server == me_server)
 		clt_ack(notice, who, AUTH_FAILED);
 	    return ZERR_NONE;
@@ -220,9 +196,6 @@ ulogin_dispatch(notice, auth, who, server)
 	}
 	return ZERR_NONE;
     } else if (strcmp(notice->z_opcode, EXPOSE_OPSTAFF) == 0) {
-#if 0
-	zdbug((LOG_DEBUG,"opstaff"));
-#endif
 	err_ret = ulogin_add_user(notice, OPSTAFF_VIS, who);
 	if (server == me_server) {
 	    if (err_ret)
@@ -231,9 +204,6 @@ ulogin_dispatch(notice, auth, who, server)
 		ack(notice, who);
 	}
     } else if (strcmp(notice->z_opcode, EXPOSE_REALMVIS) == 0) {
-#if 0
-	zdbug((LOG_DEBUG,"realmvis"));
-#endif
 	err_ret = ulogin_add_user(notice, REALM_VIS, who);
 	if (server == me_server) { /* realm vis is not broadcast,
 				      so we ack it here */
@@ -243,40 +213,36 @@ ulogin_dispatch(notice, auth, who, server)
 		ack(notice, who);
 	}
     } else if (!strcmp(notice->z_opcode, EXPOSE_REALMANN)) {
-#if 0
-	zdbug((LOG_DEBUG,"realmann"));
-#endif
 	err_ret = ulogin_add_user(notice, REALM_ANN, who);
 	if (server == me_server) { /* announce to the realm */
 	    if (err_ret)
 		nack(notice, who);
 	    else
-		login_sendit(notice, auth, who);
+		login_sendit(notice, auth, who, 0);
 	}
     } else if (!strcmp(notice->z_opcode, EXPOSE_NETVIS)) {
-#if 0
-	zdbug((LOG_DEBUG,"netvis"));
-#endif
 	err_ret = ulogin_add_user(notice, NET_VIS, who);
 	if (server == me_server) { /* announce to the realm */
 	    if (err_ret)
 		nack(notice, who);
 	    else
-		login_sendit(notice, auth, who);
+		login_sendit(notice, auth, who, 0);
 	}
     } else if (!strcmp(notice->z_opcode, EXPOSE_NETANN)) {
-#if 0
-	zdbug((LOG_DEBUG,"netann"));
-#endif
 	err_ret = ulogin_add_user(notice, NET_ANN, who);
 	if (server == me_server) { /* tell the world */
 	    if (err_ret)
 		nack(notice, who);
 	    else
-		login_sendit(notice, auth, who);
+		login_sendit(notice, auth, who, 1);
 	}
     } else {
-	syslog(LOG_ERR, "unknown ulog opcode %s", notice->z_opcode);
+	if (!strcmp(notice->z_opcode, LOGIN_USER_LOGIN)) {
+	    zdbug((LOG_DEBUG, "ulog opcode from unknown foreign realm %s",
+		   notice->z_opcode));
+	} else {
+	    syslog(LOG_ERR, "unknown ulog opcode %s", notice->z_opcode);
+	}
 	if (server == me_server)
 	    nack(notice, who);
 	return ZERR_NONE;
@@ -287,10 +253,10 @@ ulogin_dispatch(notice, auth, who, server)
 }
 
 static void
-login_sendit(notice, auth, who)
-    ZNotice_t *notice;
-    int auth;
-    struct sockaddr_in *who;
+login_sendit(ZNotice_t *notice,
+	     int auth,
+	     struct sockaddr_in *who,
+	     int external)
 {
     ZNotice_t log_notice;
 
@@ -301,7 +267,7 @@ login_sendit(notice, auth, who)
     log_notice = *notice;
 
     log_notice.z_opcode = LOGIN_USER_LOGIN;
-    sendit(&log_notice, auth, who);
+    sendit(&log_notice, auth, who, external);
 }
 
 
@@ -309,47 +275,29 @@ login_sendit(notice, auth, who)
  * Dispatch a LOCATE notice.
  */
 Code_t
-ulocate_dispatch(notice, auth, who, server)
-    ZNotice_t *notice;
-    int auth;
-    struct sockaddr_in *who;
-    Server *server;
+ulocate_dispatch(ZNotice_t *notice,
+		 int auth,
+		 struct sockaddr_in *who,
+		 Server *server)
 {
-#if 0
-    zdbug((LOG_DEBUG,"ulocate_disp"));
-#endif
+    char *cp;
+    ZRealm *realm;
 
-#if 0				/* Now we support unauthentic locate for net-visible.  */
-    if (!auth) {
-#if 0
-	zdbug((LOG_DEBUG,"unauthentic ulocate"));
-#endif
-	if (server == me_server)
-	    clt_ack(notice, who, AUTH_FAILED);
-	return ZERR_NONE;
-    }
-#endif
     if (!strcmp(notice->z_opcode, LOCATE_LOCATE)) {
-#if 0
-	zdbug((LOG_DEBUG,"locate"));
-#endif
 	/* we are talking to a current-rev client; send an ack */
 	ack(notice, who);
-	ulogin_locate(notice, who, auth);
+	cp = strchr(notice->z_class_inst, '@');
+	if (cp && (realm = realm_get_realm_by_name(cp + 1)))
+	    ulogin_locate_forward(notice, who, realm);
+	else
+	    ulogin_locate(notice, who, auth);
 	return ZERR_NONE;
     } else {
-	syslog(LOG_ERR, "unknown uloc opcode %s", notice->z_opcode);
+        syslog(LOG_ERR, "unknown uloc opcode %s", notice->z_opcode);
 	if (server == me_server)
 	    nack(notice, who);
 	return ZERR_NONE;
     }
-#if 0
-    if (server == me_server) {
-	server_forward(notice, auth, who);
-	ack(notice, who);
-    }
-    return ZERR_NONE;
-#endif
 }
 
 /*
@@ -357,8 +305,7 @@ ulocate_dispatch(notice, auth, who, server)
  */
 
 void
-uloc_hflush(addr)
-    struct in_addr *addr;
+uloc_hflush(struct in_addr *addr)
 {
     Location *loc;
     int i = 0, new_num = 0;
@@ -375,18 +322,10 @@ uloc_hflush(addr)
 
     /* copy entries which don't match */
     while (i < num_locs) {
-	if (locations[i].addr.sin_addr.s_addr != addr->s_addr) {
+	if (locations[i].addr.sin_addr.s_addr != addr->s_addr)
 	    loc[new_num++] = locations[i];
-	} else {
-#if 0
-	    if (zdebug)
-		syslog(LOG_DEBUG, "uloc hflushing %s/%s/%s",
-		       locations[i].user->string,
-		       locations[i].machine->string,
-		       locations[i].tty->string);
-#endif
+	else
 	    free_loc(&locations[i]);
-	}
 	i++;
     }
 
@@ -394,9 +333,6 @@ uloc_hflush(addr)
     locations = NULL;
 
     if (!new_num) {
-#if 0
-	zdbug((LOG_DEBUG,"no more locs"));
-#endif
 	free(loc);
 	loc = NULL;
 	num_locs = new_num;
@@ -411,8 +347,7 @@ uloc_hflush(addr)
 }
 
 void
-uloc_flush_client(sin)
-    struct sockaddr_in *sin;
+uloc_flush_client(struct sockaddr_in *sin)
 {
     Location *loc;
     int i = 0, new_num = 0;
@@ -433,13 +368,6 @@ uloc_flush_client(sin)
 	    || (locations[i].addr.sin_port != sin->sin_port)) {
 	    loc[new_num++] = locations[i];
 	} else {
-#if 0
-	    if (zdebug)
-		syslog(LOG_DEBUG, "uloc cflushing %s/%s/%s",
-		       locations[i].user->string,
-		       locations[i].machine->string,
-		       locations[i].tty->string);
-#endif
 	    free_loc(&locations[i]);
 	}
 	i++;
@@ -449,9 +377,6 @@ uloc_flush_client(sin)
     locations = NULL;
 
     if (!new_num) {
-#if 0
-	zdbug((LOG_DEBUG,"no more locs"));
-#endif
 	free(loc);
 	loc = NULL;
 	num_locs = new_num;
@@ -481,7 +406,7 @@ uloc_flush_client(sin)
 
 /*ARGSUSED*/
 Code_t
-uloc_send_locations()
+uloc_send_locations(void)
 {
     Location *loc;
     int i;
@@ -513,6 +438,7 @@ uloc_send_locations()
 	  default:
 	    syslog(LOG_ERR,"broken location state %s/%d",
 		   loc->user->string, (int) loc->exposure);
+	    exposure_level = EXPOSE_OPSTAFF;
 	    break;
 	}
 	retval = bdump_send_list_tcp(ACKED, &loc->addr, LOGIN_CLASS,
@@ -531,19 +457,24 @@ uloc_send_locations()
  */
 
 static int
-ulogin_add_user(notice, exposure, who)
-    ZNotice_t *notice;
-    Exposure_type exposure;
-    struct sockaddr_in *who;
+ulogin_add_user(ZNotice_t *notice,
+		Exposure_type exposure,
+		struct sockaddr_in *who)
 {
-    Location *oldlocs, newloc;
+    Location *loc, *oldlocs, newloc;
     int i;
 
-    if ((oldlocs = ulogin_find(notice,1)) != NULL) {
-#if 0
-	zdbug((LOG_DEBUG,"ul_add: already here"));
-#endif
-	ulogin_expose_user(notice, exposure);
+    loc = ulogin_find(notice->z_class_inst, &who->sin_addr, notice->z_port);
+    if (loc) {
+	/* Update the time, tty, and exposure on the existing location. */
+	loc->exposure = exposure;
+	if (ulogin_parse(notice, &newloc) == 0) {
+	    free_string(loc->tty);
+	    loc->tty = dup_string(newloc.tty);
+	    free(loc->time);
+	    loc->time = strsave(newloc.time);
+	    free_loc(&newloc);
+	}
 	return 0;
     }
 
@@ -595,30 +526,18 @@ ulogin_add_user(notice, exposure, who)
 	free(oldlocs);
 
   dprnt:
-#if 0
-    if (zdebug) {
-	int i;
-	syslog(LOG_DEBUG, "ul_add: New Locations (%d)", num_locs);
-	for (i = 0; i < num_locs; i++) {
-	    syslog(LOG_DEBUG, "%s/%s/%s/%d", locations[i].user->string,
-		   locations[i].machine->string, locations[i].tty->string,
-		   (int) locations[i].exposure);
-	}
-    }
-#endif
     return 0;
 }
 
 /*
  * Set up the location locs with the information in the notice.
- */ 
+ */
 
 static int
-ulogin_setup(notice, locs, exposure, who)
-    ZNotice_t *notice;
-    Location *locs;
-    Exposure_type exposure;
-    struct sockaddr_in *who;
+ulogin_setup(ZNotice_t *notice,
+	     Location *locs,
+	     Exposure_type exposure,
+	     struct sockaddr_in *who)
 {
     if (ulogin_parse(notice, locs))
 	return 1;
@@ -635,9 +554,8 @@ ulogin_setup(notice, locs, exposure, who)
  */
 
 static int
-ulogin_parse(notice, locs)
-    ZNotice_t *notice;
-    Location *locs;
+ulogin_parse(ZNotice_t *notice,
+	     Location *locs)
 {
     char *cp, *base;
     int nulls = 0;
@@ -662,122 +580,87 @@ ulogin_parse(notice, locs)
 
     cp = base;
     locs->machine = make_string(cp,0);
-#if 0
-    zdbug((LOG_DEBUG, "ul_parse: mach %s", cp));
-#endif
 
     cp += (strlen(cp) + 1);
     locs->time = strsave(cp);
-#if 0
-    zdbug((LOG_DEBUG, "ul_parse: time %s", cp));
-#endif
 
     /* This field might not be null-terminated */
     cp += (strlen(cp) + 1);
     locs->tty = make_string(cp, 0);
-#if 0
-    zdbug((LOG_DEBUG, "ul_parse: tty %s", locs->tty->string));
-#endif
 
     return 0;
-}	
+}
+
+
+static Location *
+ulogin_find(char *user,
+	    struct in_addr *host,
+	    unsigned int port)
+{
+    Location *loc;
+    String *str;
+
+    /* Find the first location for this user. */
+    loc = ulogin_find_user(user);
+    if (!loc)
+	return NULL;
+
+    /* Look for a location which matches the host and port. */
+    str = make_string(user, 0);
+    while (loc < locations + num_locs && loc->user == str) {
+	if (loc->addr.sin_addr.s_addr == host->s_addr
+	    && loc->addr.sin_port == port) {
+	    free_string(str);
+	    return loc;
+	}
+	loc++;
+    }
+
+    free_string(str);
+    return NULL;
+}
 
 /*
- * Find the username specified in notice->z_class_inst.
- * If strict, make sure the locations in notice and the table match.
- * Otherwise return a pointer to the first instance of this user@realm
- * in the table.
+ * Return a pointer to the first instance of this user@realm in the
+ * table.
  */
 
 static Location *
-ulogin_find(notice, strict)
-    ZNotice_t *notice;
-    int strict;
+ulogin_find_user(char *user)
 {
     int i, rlo, rhi;
-    Location tmploc;
     int compar;
-    String *inst;
+    String *str;
 
     if (!locations)
 	return(NULL);
 
-    inst = make_string(notice->z_class_inst, 0);
+    str = make_string(user, 0);
 
-    /* i is the current loc we are checking */
-    /* rlo is the lowest we will still check, rhi is the highest we will
-       still check */
+    /* i is the current midpoint location, rlo is the lowest we will
+     * still check, and rhi is the highest we will still check. */
 
-    i = num_locs >> 1;		/* start in the middle */
+    i = num_locs / 2;
     rlo = 0;
-    rhi = num_locs - 1;		/* first index is 0 */
+    rhi = num_locs - 1;
 
-    while ((compar = comp_string(locations[i].user, inst)) != 0) {
-#if 0
-	zdbug ((LOG_DEBUG, "ulogin_find: comparing %s %s %s %d %d",
-		notice->z_class_inst, locations[i].user->string,
-		locations[i].tty->string, rlo, rhi));
-#endif
+    while ((compar = comp_string(locations[i].user, str)) != 0) {
 	if (compar < 0)
 	    rlo = i + 1;
 	else
 	    rhi = i - 1;
 	if (rhi - rlo < 0) {
-#if 0
-	    zdbug((LOG_DEBUG,"ul_find: %s not found",
-		   inst->string));
-#endif
-	    free_string(inst);
-	    return 0;
+	    free_string(str);
+	    return NULL;
 	}
-	i = (rhi + rlo) >> 1;	/* split the diff */
+	i = (rhi + rlo) / 2;
     }
-#if 0
-    zdbug((LOG_DEBUG, "ul_find: %s found at loc %d",
-	   inst->string, i));
-#endif
-    if (strict && ulogin_parse(notice, &tmploc)) {
-#if 1
-	zdbug((LOG_DEBUG,"ul_find bad fmt"));
-#endif
-	free_string(inst);
-	return 0;
-    }
-    /* back up to the first of this guy */
-    while (i > 0 && (locations[i-1].user == inst)) {
-	i--;
-#if 0
-	zdbug((LOG_DEBUG, "ulogin_find: backing up: %s %d %s %s", inst->string,
-	       i, locations[i].user->string, locations[i].tty->string));
-#endif
-    }
-    if (strict) {
-	while (i < num_locs && !ul_equiv(&tmploc, &locations[i])
-	       && (locations[i].user == inst))
-	    i++;
-    }
-    if (strict)
-	free_loc(&tmploc);
-    if (i == num_locs || locations[i].user != inst) {
-#if 1
-	zdbug((LOG_DEBUG,"ul_find final match loss"));
-#endif
-	free_string(inst);
-	return 0;
-    }
-    free_string(inst);
-    return &locations[i];
-}
 
-static int
-ul_equiv(l1, l2)
-    Location *l1, *l2;
-{
-    if (l1->machine != l2->machine)
-	return 0;
-    if (l1->tty != l2->tty)
-	return 0;
-    return 1;
+    /* Back up to the first location for this user. */
+    while (i > 0 && locations[i - 1].user == str)
+	i--;
+    free_string(str);
+    return &locations[i];
 }
 
 /*
@@ -785,38 +668,24 @@ ul_equiv(l1, l2)
  */
 
 static Exposure_type
-ulogin_remove_user(notice, auth, who, err_return)
-    ZNotice_t *notice;
-    int auth;
-    struct sockaddr_in *who;
-    int *err_return;
+ulogin_remove_user(ZNotice_t *notice,
+		   struct sockaddr_in *who,
+		   int *err_return)
 {
     Location *new_locs, *loc;
     int i = 0;
     Exposure_type quiet;
 
     *err_return = 0;
-    loc = ulogin_find(notice, 1);
+    loc = ulogin_find(notice->z_class_inst, &who->sin_addr, notice->z_port);
     if (!loc) {
-#if 0
-	zdbug((LOG_DEBUG,"ul_rem: not here"));
-#endif
 	*err_return = NOLOC;
 	return NONE;
     }
 
-    /* if unauthentic, the sender MUST be the same IP addr that registered */
-    if (!auth && loc->addr.sin_addr.s_addr != who->sin_addr.s_addr) {
-	*err_return = UNAUTH;
-	return NONE;
-    }
-
     quiet = loc->exposure;
 
     if (--num_locs == 0) {	/* last one */
-#if 0
-	zdbug((LOG_DEBUG,"last loc"));
-#endif
 	free_loc(locations);
 	free(locations);
 	locations = NULL;
@@ -849,16 +718,6 @@ ulogin_remove_user(notice, auth, who, err_return)
 
     locations = new_locs;
 
-#if defined(DEBUG) && 0
-    if (zdebug) {
-	int i;
-
-	for (i = 0; i < num_locs; i++) {
-	    syslog(LOG_DEBUG, "%s/%d", locations[i].user->string,
-		   (int) locations[i].exposure);
-	}
-    }
-#endif
     /* all done */
     return quiet;
 }
@@ -868,20 +727,15 @@ ulogin_remove_user(notice, auth, who, err_return)
  */
 
 static void
-ulogin_flush_user(notice)
-    ZNotice_t *notice;
+ulogin_flush_user(ZNotice_t *notice)
 {
     Location *loc, *loc2;
     int i, j, num_match, num_left;
 
     i = num_match = num_left = 0;
 
-    if (!(loc2 = ulogin_find(notice, 0))) {
-#if 0
-	zdbug((LOG_DEBUG,"ul_rem: not here"));
-#endif
+    if (!(loc2 = ulogin_find_user(notice->z_class_inst)))
 	return;
-    }
 
     /* compute # locations left in the list, after loc2 (inclusive) */
     num_left = num_locs - (loc2 - locations);
@@ -894,9 +748,6 @@ ulogin_flush_user(notice)
 	num_left--;
     }
     if (num_locs == num_match) { /* no other locations left */
-#if 0
-	zdbug((LOG_DEBUG,"last loc"));
-#endif
 	for (j = 0; j < num_match; j++)
 	    free_loc(&locations[j]); /* free storage */
 	free (locations);
@@ -916,7 +767,7 @@ ulogin_flush_user(notice)
 	loc[i] = locations[i];
 	i++;
     }
-	
+
     for(j = 0; j < num_match; j++) {
 	free_loc(&locations[i]);
 	i++;
@@ -945,62 +796,11 @@ ulogin_flush_user(notice)
 #endif
 }
 
-/*
- * Set the user's exposure flag to exposure
- */
-
-static int
-ulogin_expose_user(notice, exposure)
-    ZNotice_t *notice;
-    Exposure_type exposure;
-{
-    Location *loc, loc2;
-    int idx, notfound = 1;
-
-#if 0
-    zdbug((LOG_DEBUG,"ul_expose: %s type %d", notice->z_sender,
-	   (int) exposure));
-#endif
-
-    loc = ulogin_find(notice, 0);
-    if (!loc) {
-#if 0
-		zdbug((LOG_DEBUG,"ul_hide: not here"));
-#endif
-		return 1;
-    }
-
-    if (ulogin_parse(notice, &loc2))
-	return 1;
-
-    idx = loc -locations;
-
-    while (idx < num_locs && locations[idx].user == loc2.user) {
-
-	/* change exposure and owner for each loc on that host */
-	if (locations[idx].machine == loc2.machine) {
-	    notfound = 0;
-	    locations[idx].exposure = exposure;
-	    locations[idx].addr.sin_port = notice->z_port;
-	    /* change time for the specific loc */
-	    if (locations[idx].tty == loc2.tty) {
-		free(locations[idx].time);
-		locations[idx].time = strsave(loc2.time);
-	    }
-	}
-	idx++;
-    }
-
-    free_loc(&loc2);
-    return notfound;
-}
-
 
 static void
-ulogin_locate(notice, who, auth)
-    ZNotice_t *notice;
-    struct sockaddr_in *who;
-    int auth;
+ulogin_locate(ZNotice_t *notice,
+	      struct sockaddr_in *who,
+	      int auth)
 {
     char **answer;
     int found;
@@ -1038,20 +838,20 @@ ulogin_locate(notice, who, auth)
  */
 
 static char **
-ulogin_marshal_locs(notice, found, auth)
-    ZNotice_t *notice;
-    int *found;
-    int auth;
+ulogin_marshal_locs(ZNotice_t *notice,
+		    int *found,
+		    int auth)
 {
     Location **matches = (Location **) 0;
     Location *loc;
     char **answer;
     int i = 0;
     String *inst;
+    int local = (auth && realm_sender_in_realm(ZGetRealm(), notice->z_sender));
 
     *found = 0;			/* # of matches */
 
-    loc = ulogin_find(notice, 0);
+    loc = ulogin_find_user(notice->z_class_inst);
     if (!loc)
 	return(NULL);
 
@@ -1060,16 +860,13 @@ ulogin_marshal_locs(notice, found, auth)
     inst = make_string(notice->z_class_inst,0);
     while (i < num_locs && (inst == locations[i].user)) {
 	/* these locations match */
-#if 0
-	zdbug((LOG_DEBUG,"match %s", locations[i].user->string));
-#endif
 	switch (locations[i].exposure) {
 	  case OPSTAFF_VIS:
 	    i++;
 	    continue;
 	  case REALM_VIS:
 	  case REALM_ANN:
-	    if (!auth) {
+	    if (!local) {
 		i++;
 		continue;
 	    }
@@ -1102,8 +899,8 @@ ulogin_marshal_locs(notice, found, auth)
 
     /* OK, now we have a list of user@host's to return to the client
        in matches */
-	
-	
+
+
 #ifdef DEBUG
     if (zdebug) {
 	for (i = 0; i < *found ; i++)
@@ -1111,7 +908,7 @@ ulogin_marshal_locs(notice, found, auth)
 		   matches[i]->user->string));
     }
 #endif
-	
+
     /* coalesce the location information into a list of char *'s */
     answer = (char **) malloc((*found) * NUM_FIELDS * sizeof(char *));
     if (!answer) {
@@ -1123,66 +920,54 @@ ulogin_marshal_locs(notice, found, auth)
 	    answer[i * NUM_FIELDS + 1] = matches[i]->time;
 	    answer[i * NUM_FIELDS + 2] = matches[i]->tty->string;
 	}
-	
+
     if (matches)
 	free(matches);
     return answer;
 }
 
 void
-uloc_dump_locs(fp)
-    FILE *fp;
+uloc_dump_locs(FILE *fp)
 {
     int i;
-    char buf[BUFSIZ*3];
-    static char *bufp;
-    static const char *cp;
 
-    /* delay using stdio so that we can run FAST! */
     for (i = 0; i < num_locs; i++) {
-	bufp = buf;
-#define cpy(str) cp=(str);while(*cp){*bufp++ = *cp++;}
-	cpy (locations[i].user->string);
-	*bufp++ = '/';
-	cpy (locations[i].machine->string);
-	*bufp++ = '/';
-	cpy (locations[i].time);
-	*bufp++ = '/';
-	cpy (locations[i].tty->string);
+	fputs("'", fp);
+	dump_quote(locations[i].user->string, fp);
+	fputs("' '", fp);
+	dump_quote(locations[i].machine->string, fp);
+	fputs("' '", fp);
+	dump_quote(locations[i].time, fp);
+	fputs("' '", fp);
+	dump_quote(locations[i].tty->string, fp);
+	fputs("' ", fp);
 	switch (locations[i].exposure) {
 	  case OPSTAFF_VIS:
-	    cpy ("/OPSTAFF/");
+	    fputs("OPSTAFF", fp);
 	    break;
 	  case REALM_VIS:
-	    cpy ("/RLM_VIS/");
+	    fputs("RLM_VIS", fp);
 	    break;
 	  case REALM_ANN:
-	    cpy ("/RLM_ANN/");
+	    fputs("RLM_ANN", fp);
 	    break;
 	  case NET_VIS:
-	    cpy ("/NET_VIS/");
+	    fputs("NET_VIS", fp);
 	    break;
 	  case NET_ANN:
-	    cpy ("/NET_ANN/");
+	    fputs("NET_ANN", fp);
 	    break;
 	  default:
-	    sprintf (bufp, "/? %d ?/", locations[i].exposure);
-	    while (*bufp)
-		bufp++;
+	    fprintf(fp, "? %d ?", locations[i].exposure);
 	    break;
 	}
-	cpy (inet_ntoa (locations[i].addr.sin_addr));
-	*bufp++ = '/';
-	sprintf(bufp, "%d", ntohs(locations[i].addr.sin_port));
-	fputs(buf, fp);
-	putc('\n', fp);
-#undef cpy
+	fprintf(fp, " %s/%d\n", inet_ntoa(locations[i].addr.sin_addr),
+		ntohs(locations[i].addr.sin_port));
     }
 }
 
 static void
-free_loc(loc)
-    Location *loc;
+free_loc(Location *loc)
 {
     free_string(loc->user);
     free_string(loc->machine);
@@ -1190,3 +975,102 @@ free_loc(loc)
     free(loc->time);
     return;
 }
+
+static void
+ulogin_locate_forward(ZNotice_t *notice,
+		      struct sockaddr_in *who,
+		      ZRealm *realm)
+{
+    ZNotice_t lnotice;
+
+    lnotice = *notice;
+    lnotice.z_opcode = REALM_REQ_LOCATE;
+
+    realm_handoff(&lnotice, 1, who, realm, 0);
+}
+
+void
+ulogin_realm_locate(ZNotice_t *notice,
+		    struct sockaddr_in *who,
+		    ZRealm *realm)
+{
+  char **answer;
+  int found;
+  Code_t retval;
+  ZNotice_t lnotice;
+  char *pack;
+  int packlen;
+
+#ifdef DEBUG
+  if (zdebug)
+    zdbug((LOG_DEBUG, "ulogin_realm_locate"));
+#endif
+
+  answer = ulogin_marshal_locs(notice, &found, 0/*AUTH*/);
+
+  lnotice = *notice;
+  lnotice.z_opcode = REALM_ANS_LOCATE;
+
+  if ((retval = ZFormatRawNoticeList(&lnotice, answer, found * NUM_FIELDS, &pack, &packlen)) != ZERR_NONE) {
+    syslog(LOG_WARNING, "ulog_rlm_loc format: %s",
+           error_message(retval));
+
+    if (answer)
+      free(answer);
+    return;
+  }
+  if (answer)
+    free(answer);
+
+  if ((retval = ZParseNotice(pack, packlen, &lnotice)) != ZERR_NONE) {
+    syslog(LOG_WARNING, "subscr_rlm_sendit parse: %s",
+           error_message(retval));
+    free(pack);
+    return;
+  }
+
+  realm_handoff(&lnotice, 1, who, realm, 0);
+  free(pack);
+
+  return;
+}
+
+void
+ulogin_relay_locate(ZNotice_t *notice,
+		    struct sockaddr_in *who)
+{
+  ZNotice_t lnotice;
+  Code_t retval;
+  struct sockaddr_in newwho;
+  char *pack;
+  int packlen;
+
+  notice_extract_address(notice, &newwho);
+
+  if ((retval = ZSetDestAddr(&newwho)) != ZERR_NONE) {
+    syslog(LOG_WARNING, "uloc_relay_loc set addr: %s",
+           error_message(retval));
+    return;
+  }
+
+  lnotice = *notice;
+  lnotice.z_opcode = LOCATE_LOCATE;
+  lnotice.z_kind = ACKED;
+  lnotice.z_auth = 0;
+  lnotice.z_authent_len = 0;
+  lnotice.z_ascii_authent = "";
+  lnotice.z_checksum = 0;
+  lnotice.z_ascii_checksum = "";
+
+  if ((retval = ZFormatRawNotice(&lnotice, &pack, &packlen)) != ZERR_NONE) {
+    syslog(LOG_WARNING, "ulog_relay_loc format: %s",
+           error_message(retval));
+    return;
+  }
+
+  if ((retval = ZSendPacket(pack, packlen, 0)) != ZERR_NONE) {
+    syslog(LOG_WARNING, "ulog_relay_loc xmit: %s",
+           error_message(retval));
+  }
+  free(pack);
+}
