@@ -2,9 +2,7 @@
  * Copyright (C) 1991 by the Massachusetts Institute of Technology.
  * For copying and distribution information, see the file "mit-copyright.h".
  *
- *	$Source: /mit/zephyr/repository/zephyr/server/zstring.h,v $
- *	$Id: zstring.h,v 1.4 1995/06/30 22:11:37 ghudson Exp $
- *	$Author: ghudson $
+ *	$Id: zstring.h 2527 2009-08-09 18:42:32Z kcr@ATHENA.MIT.EDU $
  */
 
 #include <zephyr/mit-copyright.h>
@@ -24,12 +22,12 @@ typedef struct _String
   struct _String *next, *prev;		/* for linking in hash table */
 } String;
 
-String *make_string __P((char *s, int downcase));
-void free_string __P((String *z));
-String *find_string __P((char *s, int downcase));
-String *dup_string __P((String *z));
-int comp_string __P((String *a, String *b));
-void print_string_table __P((FILE *f));
+String *make_string(char *s, int downcase);
+void free_string(String *z);
+String *find_string(char *s, int downcase);
+String *dup_string(String *z);
+int comp_string(String *a, String *b);
+void print_string_table(FILE *f);
 
 #endif /* __zstring_h */
 
