@@ -4,7 +4,7 @@
  *
  *      Created by:     Marc Horowitz <marc@athena.mit.edu>
  *
- *      $Id: X_gram.c 2437 2009-04-21 04:11:03Z kcr@ATHENA.MIT.EDU $
+ *      $Id: X_gram.c 2630 2011-02-02 05:26:26Z kcr@ATHENA.MIT.EDU $
  *
  *      Copyright (c) 1989 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -14,7 +14,7 @@
 #include <sysdep.h>
 
 #if (!defined(lint) && !defined(SABER))
-static const char rcsid_X_gram_c[] = "$Id: X_gram.c 2437 2009-04-21 04:11:03Z kcr@ATHENA.MIT.EDU $";
+static const char rcsid_X_gram_c[] = "$Id: X_gram.c 2630 2011-02-02 05:26:26Z kcr@ATHENA.MIT.EDU $";
 #endif
 
 #include <zephyr/mit-copyright.h>
@@ -371,7 +371,7 @@ x_gram_create(Display *dpy,
         XChangeProperty(dpy, w, net_wm_desktop, XA_CARDINAL, 32, PropModeReplace,
                         (unsigned char *) &all_desktops, 1);
 
-    XSaveContext(dpy, w, desc_context, (caddr_t)gram);
+    XSaveContext(dpy, w, desc_context, (XPointer)gram);
 
     gram->can_die.tv_sec = 0;
 
