@@ -4,7 +4,7 @@
  *
  *      Created by:     Marc Horowitz <marc@athena.mit.edu>
  *
- *      $Id: xshow.c 2441 2009-04-21 15:04:51Z kcr@ATHENA.MIT.EDU $
+ *      $Id: xshow.c 2630 2011-02-02 05:26:26Z kcr@ATHENA.MIT.EDU $
  *
  *      Copyright (c) 1989 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -14,7 +14,7 @@
 #include <sysdep.h>
 
 #if (!defined(lint) && !defined(SABER))
-static const char rcsid_xshow_c[] = "$Id: xshow.c 2441 2009-04-21 15:04:51Z kcr@ATHENA.MIT.EDU $";
+static const char rcsid_xshow_c[] = "$Id: xshow.c 2630 2011-02-02 05:26:26Z kcr@ATHENA.MIT.EDU $";
 #endif
 
 #include <zephyr/mit-copyright.h>
@@ -602,7 +602,7 @@ xhandleevent(Display *dpy,
 {
     x_gram *gram;
     
-    if (XFindContext(dpy, w, desc_context, (caddr_t *)&gram))
+    if (XFindContext(dpy, w, desc_context, (XPointer *)&gram))
       return;
 
     if (event->type == Expose)
